#!/usr/bin/env python
# coding: utf-8

# Copyright (c) TmaxEnterprise.
# Distributed under the terms of the Modified BSD License.

# Graph

import warnings

from statsmodels.formula.api import logit

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_original import StatisticOperation
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget
from jupyter_analysis_extension.statistics_widget.regression.logistic_regression import WidgetLogisticRegression


class LogisticRegression(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        # Additional init codes for specific operations.
        self.op_name = "Logistic Regression"
        self.sub = [
            {"name": "Dependent Variable", "type": ["C"], "max_count": 1, "min_count": 1},
            {"name": "Covariates", "type": ["N"], "max_count": 0, "min_count": 0},
            {"name": "Factors", "type": ["C"], "max_count": 0, "min_count": 0}
        ]
        self.option = [
            {"name": "Baseline (DV)",
             "type": "connected_unique_dropdown",
             "connect": "Dependent Variable"},
            {"name": "Baseline (Factors)",
             "type": "individual_connected_unique_dropdown",
             "connect": ["Factors"], "acc_title": "List of Factors"}
        ]
        self.callback = {"refresh": refresh_tab}

        super(LogisticRegression, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        """
        Construct the overall visual frames of linear regressions.
        Returns
        -------
        None
        """
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )
        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        """
        Perform the check of column types and the creation of analysis results of accordion widgets.
        Returns
        -------
        None
        """
        self.set_analyze_text('In progress...')

        # catch invalid data
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        covariates = []
        factors = []
        if 'Dependent Variable' in self.control_data.keys():
            dependent_var = self.column_value[self.control_data['Dependent Variable'][0]]
        if 'Covariates' in self.control_data.keys():
            for item in self.control_data['Covariates']:
                covariates.append(self.column_value[item])
        if 'Factors' in self.control_data.keys():
            for item in self.control_data['Factors']:
                factors.append(self.column_value[item])

        option = self.control_data['option']
        baseline_dv = option['Baseline (DV)']
        baseline_iv = option['Baseline (Factors)']
        result_widgets = self.get_result_widgets(dependent_var, baseline_dv, covariates, factors, baseline_iv)

        CommonUtil.update_result_layout(self.control_panel, result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):
        """
        Check the requirement numbers of columns and types.
        Returns
        -------
        str
            None if successful, error message otherwise.
        """
        error_msg = CommonUtil.check_variables_shapes(control_data, 'Dependent Variable', '2-Category',
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Covariates', 'Number',
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Factors', ['N-Category', '2-Category'],
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        if ('Covariates' not in control_data.keys()) and ('Factors' not in control_data.keys()):
            self.set_analyze_text('Please enter at least 1 column in Covariates or Factors', 'warning')
            return False


        return True

    def get_result_widgets(self, dependent_name, baseline_dv, covariates, factors, baseline_iv):
        df = self.input_df
        # Drop unrelated columns
        df = df[[dependent_name] + covariates + factors]
        # Drop the rows that have missing values in target columns.
        df = df.dropna(subset=[dependent_name] + covariates + factors)
        df[dependent_name] = (df[dependent_name] != baseline_dv).astype(int)

        y = 'Q(\"' + dependent_name + '\")'
        X = ''
        if len(covariates) > 0:
            for comp in covariates:
                x = ' Q(\"' + comp + '\")'
                X += x
                X += " +"
        else:
            pass

        if len(factors) > 0:
            for comp in factors:
                x = ' C(Q(\"' + comp + '\")'
                x += ', Treatment(reference=\'' + baseline_iv[comp] + '\'))'
                X += x
                X += " +"

        if len(X) > 2:
            X = X[:-2]
        else:
            raise Exception('No independent variables in the model')

        f = y + ' ~' + X
        with warnings.catch_warnings():
            warnings.simplefilter('ignore')
            logitfit = logit(formula=f, data=df).fit(disp=0)
        result_table = logitfit.summary2()

        # Convert column names to original
        ls = result_table.tables[1].index.to_list()
        new_ls = []
        for i in ls:
            if i[0] == 'C':
                i = i[5:]
                i = i.replace('"), Treatment', ' ')
                i = i.replace('erence', '')
                i = i.replace('))', ')')
            elif i[0] == 'Q':
                i = i[3:]
                i = i.replace('")', '')
            else:
                pass
            new_ls.append(i)
        result_table.tables[1].index = new_ls

        dep_var = result_table.tables[0].iloc[1, 1]
        dep_var = dep_var[3:]
        dep_var = dep_var.replace('")', '')
        result_table.tables[0].iloc[1, 1] = dep_var

        stat_result_table = WidgetLogisticRegression.get_statistic_result_table(result_table)

        """
        exog = sm.add_constant(np.array(df[covariates+factors]))
        endog = np.array(df[dependent_name]==baseline)

        model = sm.Logit(endog, exog)
        result = model.fit()

        stat_result_table = result.summary2()
        """
        return CommonWidget.get_accordion_widget([stat_result_table],
                                                 ['Regression Model Info'])



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) kyoungjun.
# Distributed under the terms of the Modified BSD License.
from os import name
from IPython.core.display import display
import ipywidgets as widgets

# Graph
import plotly.graph_objs as go
from plotly.tools import FigureFactory as FF
import plotly.express as px


class WidgetTTestPairSample:
    @ staticmethod
    def get_statistic_result_table(name, degree_freedom, test_result, statistic_tests_options):
        res_table_info = ''

        for pairindex, item in enumerate(test_result):
            target_degree_freedom = degree_freedom
            statistic_value = round(item.statistic, 1)
            if statistic_tests_options[pairindex] == 'Wilcoxon signed-rank':
                statistic_value = '{0:e}'.format(item.statistic)
                target_degree_freedom = '-'

            res_table_info += """
                        <tr>
                            <td>{0}</td>
                            <td>{1}</td>
                            <td>{2}</td>
                            <td>{3}</td>
                            <td>{4}</td>
                        </tr>
                        """.format(name[pairindex], statistic_tests_options[pairindex], statistic_value,
                                   target_degree_freedom, round(item.pvalue, 3))

        res_table = widgets.HTML(
            value="""
                <table style="width:100%">
                <tr style="border-bottom: 1px solid black;">
                    <th></th>
                    <th></th>
                    <th>Statistic</th>
                    <th>df</th>
                    <th>p-value</th>
                </tr>
               {0}
                </table>""".format(res_table_info),
            disabled=True
        )
        statistic_result_table = widgets.VBox([res_table])
        statistic_result_table.add_class('styledTable')

        return statistic_result_table

    @staticmethod
    def get_box_plot_plotly(data_list):
        fig = go.FigureWidget()
        for data in data_list:
            fig.add_trace(
                go.Box(y=data,
                       name=data.name,
                       boxpoints='all',
                       jitter=0.2,
                       pointpos=-1.8
                       ))

        chart_width = 300 + len(data_list)*50
        fig.update_layout(
            width=chart_width,
            height=500,
        )
        return fig



import ipywidgets as widgets

# Graph
import plotly.graph_objs as go


class WidgetBasicReliability:
    @ staticmethod
    def get_standard_table():
        standard_table = widgets.HTML(
            value="""<table style="width: 300px">
                    <thead>
                    <th>Score</th>
                    <th>Status</th>
                    </thead>
                    <tbody>
                        <tr>
                            <td>0.6 < α</td>
                            <td>No Reliable</td>
                        </tr>
                        <tr>
                            <td>0.6 > α</td>
                            <td>Acceptable</td>
                        </tr>
                        <tr>
                            <td>0.7 > α</td>
                            <td>Desirable</td>
                        </tr>
                        <tr>
                            <td>0.8 > α</td>
                            <td>High reliable</td>
                        </tr>
                        <tr>
                            <td>0.9 > α</td>
                            <td>Very reliable</td>
                        </tr>
                    </tbody>
                </table>
                <div>α = cronbach\'s value<div>
                """,
            disabled=True
        )
        standard_table.add_class('styledTable')

        return standard_table

    @staticmethod
    def get_box_plot_plotly(data_list):
        fig = go.FigureWidget()
        for data in data_list:
            fig.add_trace(
                go.Box(y=data,
                       name=data.name,
                       boxpoints='all',
                       jitter=0.2,
                       pointpos=-1.8
                       ))
        fig.update_layout(
            title='Box plot',
            title_x=0.5,
            title_font_size=25,
            width=500,
            height=500,
            hovermode='closest',
        )
        return fig

    @staticmethod
    def get_headmap_plotly(data, columns):
        fig = go.FigureWidget()
        fig.add_trace(go.Heatmap(x=columns, y=columns, z=data.corr()))
        fig.update_layout(
            title='Heatmap',
            title_x=0.5,
            title_font_size=25,
            width=500,
            height=500,
            hovermode='closest',
        )
        return fig


#!/usr/bin/env python
# coding: utf-8

# Copyright (c) kyoungjun.
# Distributed under the terms of the Modified BSD License.
import pandas as pd
from statsmodels.stats.contingency_tables import mcnemar

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_original import StatisticOperation
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget
from jupyter_analysis_extension.statistics_widget.frequencies.frequencies_mcnemar import WidgetFrequenciesMcnemar


class FrequenciesMcnemar(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        self.op_name = "McNemar test"
        self.sub = [
            {"name": "Rows", "type": ["N"], "max_count": 1, "min_count": 1},
            {"name": "Columns", "type": ["N"], "max_count": 1, "min_count": 1},
        ]
        self.option = [
            {"name": "Test Types", "type": "checkbox", "value": ['Mcnemar', 'Continuity correction']}
        ]
        self.callback = {"refresh": refresh_tab}
        super(FrequenciesMcnemar, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )

        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        self.set_analyze_text('In progress...')

        # catch invalid data
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        # get analysis data & option

        rows_layout_var_list = list(self.control_data['Rows'])
        rows_var_list = []
        for layout_var in rows_layout_var_list:
            rows_var_list.append(self.column_value[layout_var])

        columns_layout_var_list = list(self.control_data['Columns'])
        columns_var_list = []
        for layout_var in columns_layout_var_list:
            columns_var_list.append(self.column_value[layout_var])

        option = self.control_data['option']
        tests_chi = option['Test Types']['Mcnemar']
        tests_fisher = option['Test Types']['Continuity correction']

        result_widgets = self.get_result_widgets(rows_var_list, columns_var_list, [tests_chi, tests_fisher])

        CommonUtil.update_result_layout(self.control_panel, result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):

        error_msg = CommonUtil.check_option_checkbox(control_data, 'Test Types', 1)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Rows', ['2-Category'],
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Columns', ['2-Category'],
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        return True

    def get_result_widgets(self, rows_var_list, columns_var_list, tests_options):
        first_col_values = []
        statistic_tests_options = []
        statistic_result = {}

        cross_tab = pd.crosstab(self.input_df[rows_var_list[0]], self.input_df[columns_var_list[0]])

        if tests_options[0]:
            statistic_result["Mcnemar"] = mcnemar(cross_tab, exact=False, correction=False)
            first_col_values.append("Mcnemar")

        if tests_options[1]:
            statistic_result["Continuity correction"] = mcnemar(cross_tab, exact=False, correction=True)
            first_col_values.append("Continuity correction")

        cross_tab = pd.crosstab(self.input_df[rows_var_list[0]], self.input_df[columns_var_list[0]], margins=True,
                                margins_name='Total')
        df_count = len(self.input_df)

        statistic_result_table = WidgetFrequenciesMcnemar.get_statistic_result_table(first_col_values, df_count,
                                                                                     statistic_result,
                                                                                     statistic_tests_options)

        data_table = WidgetFrequenciesMcnemar.get_data_table(cross_tab)

        return CommonWidget.get_accordion_widget([statistic_result_table, data_table], ['Test Result(s)', 'Data table'])



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) kyoungjun.
# Distributed under the terms of the Modified BSD License.
import json
import logging
from itertools import combinations
import pingouin as pg

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_op import StatisticOperation
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget

# Graph
from jupyter_analysis_extension.statistics_widget.regression.partial_correlation import WidgetPartialCorrelation
from jupyter_analysis_extension.utils.preprocess import preprocess_values


class PartialCorrelation(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        # Additional init codes for specific operations.
        self.op_name = "Partial Correlation"
        self.sub = [
            {"name": "Variables", "type": ["N"], "max_count": 0, "min_count": 2},
            {"name": "Control Variables", "type": ["N"], "max_count": 0, "min_count": 1}
        ]
        self.option = [
            {"name": "Correlation Coefficients", "type": "checkbox",
             "value": ["Pearson", "Spearman"]},
            {"name": "Correlation Type", "type": "radio",
             "value": ["Partial", "Semi-partial"]},
            # Semi-partial: compute semi-partial based on x variable not y. (x_cover: o, y_cover: x in pingouin)
            {"name": "Additional Options", "type": "checkbox",
             "value": ["Report significance (p-value)"]}
        ]
        self.callback = {"refresh": refresh_tab}

        super(PartialCorrelation, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        """
        Construct the overall visual frames of partial correlation.
        Returns
        -------
        None
        """
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )
        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        """
        Perform the check of column types and the creation of analysis results of accordion widgets.
        Returns
        -------
        None
        """
        self.set_analyze_text('In progress...')

        # catch invalid data
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        obj_vars = None
        if 'Variables' in self.control_data.keys():
            obj_vars = []
            for item in self.control_data['Variables']:
                obj_vars.append(self.column_value[item])

        control_vars = None
        if 'Control Variables' in self.control_data.keys():
            control_vars = []
            for item in self.control_data['Control Variables']:
                control_vars.append(self.column_value[item])

        option = self.control_data['option']
        corr_coefficients = [x for x in option["Correlation Coefficients"].keys() if
                             option["Correlation Coefficients"][x] is True]
        addi_ops = [x for x in option["Additional Options"].keys() if option["Additional Options"][x] is True]
        corr_type = option["Correlation Type"]['radio']

        self.result_widgets = self.get_result_widgets(obj_vars, control_vars, corr_coefficients, corr_type, addi_ops)

        CommonUtil.update_result_layout(self.control_panel, self.result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):
        """
        Check the requirement numbers of columns and types.
        Returns
        -------
        str
            None if successful, error message otherwise.
        """
        error_msg = CommonUtil.check_variables_shapes(control_data, 'Variables', 'Number',
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Control Variables', 'Number',
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        return True

    def get_result_widgets(self, var_names, control_names, corr_coefficients, corr_type, addi_ops):
        # self.logger.info(corr_coefficients)
        # self.logger.info(addi_ops)
        # self.logger.info(corr_type)

        plot_list, title_list = self.get_partial_correlation_info(var_names, control_names, corr_coefficients,
                                                                  corr_type, addi_ops)

        # Set the info widgets for exporting
        self.info_widgets = CommonWidget.get_accordion_widget(plot_list, title_list)

        # Export Info
        # HTML
        export_html_button = CommonWidget.get_button_with_icon('export-html')
        export_html_button.on_click(self.on_export_html_button_clicked)

        # PDF
        # export_pdf_button = CommonWidget.get_button_with_icon('export-pdf')
        # export_pdf_button.on_click(self.on_export_pdf_button_clicked)

        # export_layout = CommonWidget.get_export_layout(export_html_button, export_pdf_button)
        export_layout = CommonWidget.get_export_layout(export_html_button)
        plot_list.append(export_layout)

        title_list.append("Export Results")
        return CommonWidget.get_accordion_widget(plot_list, title_list)

        # self.result_widgets = CommonWidget.add_export_layout_on_accordion(self.result_widgets, export_html_button)

    def get_partial_correlation_info(self, var_names, control_names, corr_coefficients, corr_type, addi_ops):
        result_plot = []
        result_title = []

        EDA_plot_title = "Exploratory Data Analysis (EDA)"
        sampled_df = self.input_df.copy()
        if len(self.input_df) > 10000:
            EDA_plot_title += " (Sampling 10000 rows)"
            # Sampling only 10000 rows
            sampled_df = sampled_df.sample(n=10000, random_state=1, replace=False)
        # EDA Plot
        eda_plot = WidgetPartialCorrelation.get_all_scatter_plot(sampled_df, var_names, control_names)

        result_plot.append(eda_plot)
        result_title.append(EDA_plot_title)

        # Corr Info
        corr_info = {}
        for combo in combinations(var_names, 2):
            df = self.input_df[[combo[0], combo[1]] + control_names]
            df = preprocess_values(df)

            # x = np.array(df[combo[0]])
            # y = np.array(df[combo[1]])
            corr_info[combo] = {}
            if corr_type == "Partial":
                if "Pearson" in corr_coefficients:
                    corr_info[combo]["Pearson(n)"], corr_info[combo]["Pearson(r)"], corr_info[combo]["Pearson(CI95)"], corr_info[combo]["Pearson(p)"] \
                        = pg.partial_corr(data=df, x=combo[0], y=combo[1], covar=control_names, method='pearson').round(3).iloc[0].values
                if "Spearman" in corr_coefficients:
                    corr_info[combo]["Spearman(n)"], corr_info[combo]["Spearman(r)"], corr_info[combo]["Spearman(CI95)"], corr_info[combo]["Spearman(p)"] \
                        = pg.partial_corr(data=df, x=combo[0], y=combo[1], covar=control_names, method='spearman').round(3).iloc[0].values
            elif corr_type == "Semi-partial":
                if "Pearson" in corr_coefficients:
                    corr_info[combo]["Pearson(n)"], corr_info[combo]["Pearson(r)"], corr_info[combo]["Pearson(CI95)"], corr_info[combo]["Pearson(p)"] \
                        = pg.partial_corr(data=df, x=combo[0], y=combo[1], x_covar=control_names, method='pearson').round(3).iloc[0].values
                if "Spearman" in corr_coefficients:
                    corr_info[combo]["Spearman(n)"], corr_info[combo]["Spearman(r)"], corr_info[combo]["Spearman(CI95)"], corr_info[combo]["Spearman(p)"] \
                        = pg.partial_corr(data=df, x=combo[0], y=combo[1], x_covar=control_names, method='spearman').round(3).iloc[0].values
            else:
                self.logger.error("Do not support type: %s", corr_type)
                exit(1)
        stat_result_table = WidgetPartialCorrelation.get_statistic_result_table(corr_info, corr_coefficients, addi_ops)

        result_plot.append(stat_result_table)
        result_title.append("Partial Correlation")

        return result_plot, result_title



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) kyoungjun.
# Distributed under the terms of the Modified BSD License.

from jupyter_analysis_extension.example import ExampleWidget
from jupyter_analysis_extension.widget_func import WidgetFunc



def test_example_creation_blank():
    w = ExampleWidget()
    assert w.value == 'Hello World'



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) kyoungjun.
# Distributed under the terms of the Modified BSD License.
import json
import logging
from itertools import combinations
import numpy as np
from ipywidgets import IntSlider
from ipywidgets.embed import embed_minimal_html, embed_data, dependency_state
from scipy.stats import pearsonr, spearmanr, kendalltau

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_op import StatisticOperation
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget

# Graph
from jupyter_analysis_extension.statistics_widget.regression.correlation_matrix import WidgetCorrelationMatrix
from jupyter_analysis_extension.utils.preprocess import preprocess_values


class CorrelationMatrix(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        # Additional init codes for specific operations.
        self.op_name = "Correlation Matrix"
        self.sub = [
            {"name": "Variables", "type": ["N"], "max_count": 0, "min_count": 2}
        ]
        self.option = [
            {"name": "Correlation Coefficients", "type": "checkbox",
             "value": ["Pearson", "Spearman", "Kendall's tau-b"]},
            {"name": "Additional Options", "type": "checkbox",
             "value": ["Report significance (p-value)"]}
        ]
        self.callback = {"refresh": refresh_tab}

        super(CorrelationMatrix, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        """
        Construct the overall visual frames of correlation matrix.
        Returns
        -------
        None
        """
        #                 {"name": "Hypothesis", "type": "radio",
        #                  "value": ["Correlated", "Correlated positively", "Correlated negatively"]}
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )
        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        """
        Perform the check of column types and the creation of analysis results of accordion widgets.
        Returns
        -------
        None
        """
        self.set_analyze_text('In progress...')

        # catch invalid data
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        obj_vars = None
        if 'Variables' in self.control_data.keys():
            obj_vars = []
            for item in self.control_data['Variables']:
                obj_vars.append(self.column_value[item])

        option = self.control_data['option']
        corr_coefficients = [x for x in option["Correlation Coefficients"].keys() if
                             option["Correlation Coefficients"][x] is True]
        addi_ops = [x for x in option["Additional Options"].keys() if option["Additional Options"][x] is True]
        # hypo = option["Hypothesis"]['radio']

        self.result_widgets = self.get_result_widgets(obj_vars, corr_coefficients, addi_ops)

        CommonUtil.update_result_layout(self.control_panel, self.result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):
        """
        Check the requirement numbers of columns and types.
        Returns
        -------
        str
            None if successful, error message otherwise.
        """
        error_msg = CommonUtil.check_variables_shapes(control_data, 'Variables', 'Number',
                                                      self.column_type_list, self.column_value, self.sub)

        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        return True

    def get_result_widgets(self, var_names, corr_coefficients, addi_ops):
        # self.logger.info(corr_coefficients)
        # self.logger.info(addi_ops)
        # self.logger.info(hypo)

        plot_list, title_list = self.get_correlation_matrix_info(var_names, corr_coefficients, addi_ops)

        # Set the info widgets for exporting
        self.info_widgets = CommonWidget.get_accordion_widget(plot_list, title_list)

        # Export Info
        # release 1.0.0 대비 비활성화 처리
        # # HTML
        # export_html_button = CommonWidget.get_button_with_icon('export-html')
        # export_html_button.on_click(self.on_export_html_button_clicked)
        #
        # # PDF
        # # export_pdf_button = CommonWidget.get_button_with_icon('export-pdf')
        # # export_pdf_button.on_click(self.on_export_pdf_button_clicked)
        #
        # # export_layout = CommonWidget.get_export_layout(export_html_button, export_pdf_button)
        # export_layout = CommonWidget.get_export_layout(export_html_button)
        # plot_list.append(export_layout)
        #
        # title_list.append("Export Results")
        return CommonWidget.get_accordion_widget(plot_list, title_list)

        # self.result_widgets = CommonWidget.add_export_layout_on_accordion(self.result_widgets, export_html_button)

    def get_correlation_matrix_info(self, var_names, corr_coefficients, addi_ops):
        result_plot = []
        result_title = []

        EDA_plot_title = "Exploratory Data Analysis (EDA)"
        sampled_df = self.input_df.copy()
        if len(self.input_df) > 10000:
            EDA_plot_title += " (Sampling 10000 rows)"
            # Sampling only 10000 rows
            sampled_df = sampled_df.sample(n=10000, random_state=1, replace=False)
        # EDA Plot
        eda_plot = WidgetCorrelationMatrix.get_all_scatter_plot(sampled_df, var_names)

        result_plot.append(eda_plot)
        result_title.append(EDA_plot_title)

        # Corr Info
        corr_info = {}
        for combo in combinations(var_names, 2):
            df = self.input_df[[combo[0], combo[1]]]
            df = preprocess_values(df)

            x = np.array(df[combo[0]])
            y = np.array(df[combo[1]])

            corr_info[combo] = {}
            if "Pearson" in corr_coefficients:
                corr_info[combo]["Pearson(corr)"], corr_info[combo]["Pearson(p)"] = pearsonr(x, y)
                corr_info[combo]["Pearson(corr)"] = round(corr_info[combo]["Pearson(corr)"], 4)
                corr_info[combo]["Pearson(p)"] = "< 0.001" if corr_info[combo]["Pearson(p)"] < 0.001 else \
                    round(corr_info[combo]["Pearson(p)"], 4)
            if "Spearman" in corr_coefficients:
                corr_info[combo]["Spearman(corr)"], corr_info[combo]["Spearman(p)"] = spearmanr(x, y)
                corr_info[combo]["Spearman(corr)"] = round(corr_info[combo]["Spearman(corr)"], 4)
                corr_info[combo]["Spearman(p)"] = "< 0.001" if corr_info[combo]["Spearman(p)"] < 0.001 else \
                    round(corr_info[combo]["Spearman(p)"], 4)
            if "Kendall's tau-b" in corr_coefficients:
                corr_info[combo]["Kendall's tau-b(corr)"], corr_info[combo]["Kendall's tau-b(p)"] = kendalltau(x, y)
                corr_info[combo]["Kendall's tau-b(corr)"] = round(corr_info[combo]["Kendall's tau-b(corr)"], 4)
                corr_info[combo]["Kendall's tau-b(p)"] = "< 0.001" if corr_info[combo]["Kendall's tau-b(p)"] < 0.001 \
                    else round(corr_info[combo]["Kendall's tau-b(p)"], 4)

        stat_result_table = WidgetCorrelationMatrix.get_statistic_result_table(corr_info, corr_coefficients, addi_ops)

        result_plot.append(stat_result_table)
        result_title.append("Correlation Matrix")

        return result_plot, result_title



import numpy as np

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_original import StatisticOperation
from jupyter_analysis_extension.statistics_widget.basic.basic_reliability import WidgetBasicReliability
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget
from jupyter_analysis_extension.utils.widget_maker import WidgetMaker


class BasicReliability(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        self.op_name = "Reliability Analysis"
        self.sub = [
            {"name": "Group", "type": ["N"], "max_count": 0, "min_count": 1}
        ]
        self.option = [
            {"name": "Statistical table", "type": "checkbox", "value": ['Cronbach\'s', 'Standard table']},
            {"name": "Chart types", "type": "checkbox", "value": ['Box plot', 'Headmap']},
        ]
        self.callback = {"refresh": refresh_tab}
        super(BasicReliability, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )

        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        self.set_analyze_text('In progress...')

        # catch invalid data
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        # get analysis data & option

        group_layout_var_list = list(self.control_data['Group'])
        group_var_list = []
        for layout_var in group_layout_var_list:
            group_var_list.append(self.column_value[layout_var])

        option = self.control_data['option']
        option_statistical_table = option['Statistical table']
        option_chart_types = option['Chart types']

        result_widgets = self.get_result_widgets(group_var_list, option_statistical_table, option_chart_types)

        CommonUtil.update_result_layout(self.control_panel, result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):

        error_msg = CommonUtil.check_option_checkbox(control_data, 'Statistical table', 1)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_option_checkbox(control_data, 'Chart types', 1)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Group', ['Number'],
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        return True

    def get_result_widgets(self, group_var_list, option_statistical_table, option_chart_types):
        cronbach_alpha_group_list = []

        def cronbach_alpha_group(group_items):
            group_items = np.asarray(group_items)
            items_var = group_items.var(axis=1, ddof=1)
            items_sum = group_items.sum(axis=0)
            items_len = len(group_items)
            return items_len / (items_len - 1.) * (1 - items_var.sum() / items_sum.var(ddof=1))

        for group_var in group_var_list:
            cronbach_alpha_group_list.append(self.input_df[group_var])

        statistic_table_header = ['', 'Cronbach\'s', 'Mean', 'Deviation']
        statistic_table_body = [['Group', round(cronbach_alpha_group(cronbach_alpha_group_list), 3), '-', '-']]
        statistic_table_footer = "* Cronbach\'s > 0.6 is Reliable"
        for group_var in group_var_list:
            target_df = self.input_df[group_var]
            statistic_table_body.append([" - " + group_var, '-', round(target_df.mean(), 3), round(target_df.std(), 3)])

        statistic_table = WidgetMaker.get_styled_table(statistic_table_header, statistic_table_body,
                                                       statistic_table_footer)

        if option_statistical_table['Standard table']:
            standard_table = WidgetBasicReliability.get_standard_table()
            table_children = list(statistic_table.children)
            table_children.append(standard_table)
            statistic_table.children = tuple(table_children)

        chart_accordion_title = "Chart"
        statistic_chart = []
        for group_var in group_var_list:
            statistic_chart.append(self.input_df[group_var])

        if len(self.input_df) > 10000:
            # Sampling only 10000 rows
            chart_accordion_title += " (Sampling 10000 rows)"
            for index, df in enumerate(statistic_chart):
                statistic_chart[index] = df.copy().sample(n=10000, random_state=1, replace=False)

        chart_list = []
        if option_chart_types['Box plot']:
            chart_list.append(WidgetBasicReliability.get_box_plot_plotly(statistic_chart))
        if option_chart_types['Headmap']:
            chart_list.append(WidgetBasicReliability.get_headmap_plotly(self.input_df, group_var_list))
        chart_accordion_item = WidgetMaker.get_vertical_box(box_items=chart_list,
                                                            style=dict(display="flex", width="100%",
                                                                       align_items="center"))

        return CommonWidget.get_accordion_widget([statistic_table, chart_accordion_item],
                                                 ['Statistical table', chart_accordion_title])



from xhtml2pdf import pisa

HTML_TEMPLATE = """
<html>
  <head>

    <title>{title}</title>

    <!-- Load RequireJS, used by the IPywidgets for dependency management -->
    <script 
      src="https://cdnjs.cloudflare.com/ajax/libs/require.js/2.3.4/require.min.js" 
      integrity="sha256-Ae2Vz/4ePdIu6ZyI/5ZGsYnb+m0JlOmKPjt6XZ9JJkA=" 
      crossorigin="anonymous">
    </script>

    <!-- Load IPywidgets bundle for embedding. -->
    <script
      data-jupyter-widgets-cdn="https://unpkg.com/"
      data-jupyter-widgets-cdn-only
      src="https://cdn.jsdelivr.net/npm/@jupyter-widgets/html-manager@*/dist/embed-amd.js" 
      crossorigin="anonymous">
    </script>

    <!-- The state of all the widget models on the page -->
    <script type="application/vnd.jupyter.widget-state+json">
      {manager_state}
    </script>
  </head>

  <body>

    <h1 align="center">{title}</h1>

    {widget_views}

  </body>
</html>
"""

WIDGET_VIEW_TEMPLATE = """
    <h2>{title}</h2>
    <div id="{num}-slider-widget">
      <!-- This script tag will be replaced by the view's DOM tree -->
      <script type="application/vnd.jupyter.widget-view+json">
        {widget_view}
      </script>
    </div>

    <hrule />
"""


def convert_html_to_pdf(source_html, output_filename):
    # open output file for writing (truncated binary)
    result_file = open(output_filename, "w+b")

    # convert HTML to PDF
    pisa_status = pisa.CreatePDF(
            source_html,                # the HTML to convert
            dest=result_file)           # file handle to receive result

    # close output file
    result_file.close()                 # close output file

    # return True on success and False on errors
    return pisa_status.err



import json
import logging
import os
import webbrowser
from abc import *

import pdfkit
import imgkit
from ipywidgets.embed import dependency_state, embed_data
import ipywidgets as widgets
from jupyter_analysis_extension.utils.export_info import HTML_TEMPLATE, WIDGET_VIEW_TEMPLATE, convert_html_to_pdf
from jupyter_analysis_extension.utils.logger import OutputWidgetHandler


class StatisticOperation:
    """operation default class"""
    # Type Hint
    info_widgets: widgets.Accordion()
    result_widgets: widgets.Accordion()
    logger: logging.Logger

    # main integrated widget
    control_panel = None

    def __init__(self, op_name, df, column_type_list, refresh_tab, logger):
        # Data
        self.info_widgets = None
        self.result_widgets = None
        self.input_df = df
        self.column_type_list = column_type_list
        self.op_name = op_name

        self.column_value = {}
        self.control_data = None

        # Widget & Layout
        self.control_panel = None
        self.analyze_layout = None
        self.analyze_textfield = None
        self.refresh_textfield = None

        # func
        self.refresh_tab = refresh_tab

        # log
        self.logger = logger
        self.construct_control_panel()

    @abstractmethod
    def construct_control_panel(self):
        pass

    @abstractmethod
    def on_analyze_button_clicked(self, widget):
        pass

    def refresh_tab(self):
        if self.analyze_textfield is not None:
            self.analyze_textfield.value = ""

        if self.refresh_textfield is not None:
            self.refresh_textfield.value = ""
        return

    def get_tab_widget(self):
        return self.control_panel

    def connect_button_layout(self, button_layout, analyze_btn_event):
        """

        Parameters
        ----------
        button_layout
            button-textField widget layout
        analyze_btn_event
            trigger analyze btn event
        -------
        """
        button_layout = button_layout
        self.analyze_textfield = button_layout.children[0].children[0]
        self.refresh_textfield = button_layout.children[0].children[1]
        button_layout.children[1].children[0].on_click(analyze_btn_event)

    def set_analyze_text(self, text, mode="plain"):
        """

        Parameters
        ----------
        text
            real textField string
        mode
            plain | warning
        """
        target_text = """<div style="display:flex;justify-content:flex-end">{0}</div>""".format(
            text)
        if mode == "warning":
            target_text = """<div style="font-size:1.5rem;color:red;display:flex;justify-content:flex-end">{0}</div>""".format(
                text)
        self.analyze_textfield.value = target_text

    def set_refresh_text(self, text):
        """

        Parameters
        ----------
        text
            real textField string
        """
        if hasattr(self, "refresh_textfield"):
            self.refresh_textfield.value = """<div style="font-size:1.5rem;color:red;display:flex;justify-content:flex-end">{0}</div>""".format(
                text)

    def on_export_html_button_clicked(self, widget):
        self.create_html_file(file_name=self.op_name + ".html")

    def on_export_pdf_button_clicked(self, widget):
        self.create_html_file(file_name=self.op_name + ".html", open_tab=False)
        # convert_html_to_pdf(self.op_name + ".html", self.op_name + ".pdf")
        imgkit.from_file(self.op_name + ".html", self.op_name + ".png")
        os.remove(self.op_name + ".html")
        webbrowser.open_new_tab(self.op_name + ".png")

    def create_html_file(self, file_name, open_tab=True):
        # Simple version
        # embed_minimal_html('export2.html', views=widget, state=dependency_state(widget))

        # Customized version (**)
        data = embed_data(views=self.info_widgets.children)

        manager_state = data['manager_state']
        manager_state["state"] = dependency_state(self.info_widgets.children)
        widget_views = [json.dumps(view) for view in data['view_specs']]
        widget_agg_views = ""
        for i, widget_view in enumerate(widget_views):
            widget_agg_views += WIDGET_VIEW_TEMPLATE.format(title=self.info_widgets.get_title(i), num=i,
                                                            widget_view=widget_view)

        rendered_template = HTML_TEMPLATE.format(title=self.op_name, manager_state=json.dumps(manager_state),
                                                 widget_views=widget_agg_views)
        with open(file_name, 'w') as fp:
            fp.write(rendered_template)

        if open_tab:
            webbrowser.open_new_tab(file_name)




#!/usr/bin/env python
# coding: utf-8

# Copyright (c) TmaxBI.
# Distributed under the terms of the Modified BSD License.

import statsmodels.api as sm
from statsmodels.formula.api import ols

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_original import StatisticOperation
from jupyter_analysis_extension.statistics_widget.anova.anova_ancova import WidgetANOVAANCOVA
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget


class ANOVAANCOVA(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        # Additional init codes for specific operations.
        self.op_name = "ANOVA & ANCOVA"
        self.sub = [
            {"name": "Dependent Variable", "type": ["N"], "max_count": 1, "min_count": 1},
            {"name": "Fixed Factors", "type": ["C"], "max_count": 3, "min_count": 1},
            {"name": "Covariates", "type": ["N"], "max_count": 0}
        ]
        self.option = [
            {"name": "Type", "type": "radio", "value": ['Type 1', 'Type 2', 'Type 3']},
            {"name": "Model-Fit", "type": "connected_multiple", "acc_title": "Model",
             "value": ["Components", "Model Terms"], "connect": ["Fixed Factors", "Covariates"]},
        ]
        self.callback = {"refresh": refresh_tab}

        super(ANOVAANCOVA, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):

        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )

        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        self.set_analyze_text('In progress...')

        # catch invalid data
        option = self.control_data['option']
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        # get analysis data & option
        degree_freedom = len(self.input_df) - 2

        dependent_var = self.control_data['Dependent Variable']

        model_terms_list = list(option['Model Terms'])

        test_type = option['Type']['radio']
        test_type = int(test_type[-1])

        result_widgets = self.get_result_widgets(
            degree_freedom, dependent_var, model_terms_list, test_type)

        CommonUtil.update_result_layout(self.control_panel, result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Dependent Variable', 'Number',
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Covariates', 'Number',
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Fixed Factors', ['N-Category', '2-Category'],
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        if "Model Terms" not in control_data['option'] or len(list(control_data['option']['Model Terms'])) == 0:
            self.set_analyze_text('Option[Model-Fit] Model Terms should have at least 1 column', "warning")
            return False

        return True

    def get_result_widgets(self, degree_freedom, dependent_var, model_terms_list, test_type):
        interaction_mark = ' ※ '

        dep_var = self.column_value[dependent_var[0]]
        statistic_chart = []
        statistic_chart_title = []
        statistic_result = []
        df = self.input_df
        y = 'Q(\"' + dep_var + '\")'
        X = ''
        for comp in model_terms_list:
            if interaction_mark in comp:
                ls = comp.split(interaction_mark)
                x = ' C(Q(\"' + ls[0] + '\"))' + '*' + 'C(Q(\"' + ls[1] + '\"))'
            else:
                if self.column_type_list[comp] == 'Number':
                    x = ' Q(\"' + comp + '\")'
                elif self.column_type_list[comp] == 'N-Category':
                    x = ' C(Q(\"' + comp + '\"))'
                elif self.column_type_list[comp] == '2-Category':
                    x = ' C(Q(\"' + comp + '\"))'
                    if len(df[comp].dropna().unique()) == 1:
                        df[comp] = df[comp].fillna('null')

            X += x
            X += " +"

        X = X[:-2]

        formula = y + ' ~' + X
        model = ols(formula, data=df).fit()

        table = sm.stats.anova_lm(model, typ=test_type)
        # Convert column names to original
        ls = table.index.to_list()
        new_ls = []
        for i in ls:
            if i[0] == 'C':
                i = i[5:]
                i = i.replace('"))', '')
            elif i[0] == 'Q':
                i = i[3:]
                i = i.replace('")', '')
            else:
                pass
            new_ls.append(i)
        table.index = new_ls
        statistic_result_table = WidgetANOVAANCOVA.get_statistic_result_table(table, degree_freedom, test_type)

        ## TODO. Which plot to draw for ANOVA & ANCOVA?
        """
        boxplot_sns = WidgetANOVAANCOVA.get_boxplot_sns()
        statistic_chart.append(boxplot_sns)
        statistic_chart_title.append('Box Plot (plotly): ' + str(dep))
        """

        return CommonWidget.get_accordion_widget([statistic_result_table, *statistic_chart],
                                                 ['Test Result(s)', *statistic_chart_title])

    """

    def Ftest(df, dim, mes):
        dim = 'C(Q(\"' + dim + '\"))'
        mes = 'Q(\"' + mes + '\")'
        equation = mes + ' ~ ' + dim
        try:
            model = ols(equation, data=df).fit()
        except Exception as e:
            return np.nan, 1.
        table = sm.stats.anova_lm(model, typ=2)
        table['mean_sq'] = table[:]['sum_sq'] / table[:]['df']
        table['eta_sq'] = table[:-1]['sum_sq'] / sum(table['sum_sq'])
        table['omega_sq'] = (table[:-1]['sum_sq'] - (table[:-1]['df'] * table['mean_sq'][-1])) / (
                sum(table['sum_sq']) + table['mean_sq'][-1])
        p_value = table.at[dim, 'PR(>F)']
        eta_sq = table.at[dim, 'eta_sq']
        omega_sq = np.abs(table.at[dim, 'omega_sq'])
        effect_size = omega_sq ** (0.4 - (2 * omega_sq)) if omega_sq <= 0.2 else 1.0
        return effect_size, p_value
    """



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) kyoungjun.
# Distributed under the terms of the Modified BSD License.
import ipywidgets as widgets
import matplotlib.pyplot as plt
# Graph
import plotly.graph_objs as go
import seaborn as sns
from scipy import stats


class WidgetTTest:
    def __init__(self, df):
        self.input_df = df

        self.group_sub = widgets.Dropdown(
            options=self.input_df.columns,
            description='Grouping Subject: ',
            style=dict(description_width='initial')
        )
        self.column = widgets.Dropdown(
            options=self.input_df.columns,
            description='Column: ',
            style=dict(description_width='initial')
        )
        click_button = widgets.Button(
            description='Analyze',
            disabled=False,
            button_style='',
            tooltip='Analyze',
            icon='check')

        click_button.on_click(self.on_analyze_button_clicked)
        margin = widgets.Layout(margin='5px 5px 5px 5px', align_items='center')
        self.agg_widget = widgets.VBox(
            [widgets.HBox([self.group_sub, self.column, click_button], layout=margin)], layout=margin)

    def get_t_test_intro_widget(self):
        return self.agg_widget

    def on_analyze_button_clicked(self, widget):
        sub1 = self.input_df.groupby(self.group_sub.value)[
            self.column.value].nth(0)
        sub2 = self.input_df.groupby(self.group_sub.value)[
            self.column.value].nth(1)

        self.agg_widget.children += (self.get_accordion_widgets(sub1, sub2),)
        # self.agg_widget.children = (*self.agg_widget.children, sharpiro_agg_widget)

    def get_accordion_widgets(self, sub1, sub2):
        sharpiro_agg_widget = self.get_shapiro_widgets(sub1, sub2)
        int_test_ind_widget = self.get_int_test_ind_widget(sub1, sub2)
        box_plot_plotly_widget = self.get_box_plot_plotly(sub1, sub2)
        box_plot_seaborn_widget = self.get_box_plot_seaborn(
            self.input_df, self.group_sub.value, self.column.value)
        accordion_widget = widgets.Accordion(children=[sharpiro_agg_widget, int_test_ind_widget,
                                                       box_plot_plotly_widget, box_plot_seaborn_widget], layout=widgets.Layout(width='100%'))
        title_list = ['Sharpiro', 'INT IND',
                      'Box Plot (plotly)', 'Box Plot (seaborn)']
        for i in range(len(accordion_widget.children)):
            accordion_widget.set_title(i, title_list[i])
        return accordion_widget

    @staticmethod
    def get_box_plot_plotly(sub1, sub2):
        f = go.FigureWidget()
        f.add_trace(go.Box(y=sub1))
        f.add_trace(go.Box(y=sub2))
        return f

    @staticmethod
    def get_box_plot_seaborn(input_df, group_sub, column):
        out = widgets.Output()
        with out:
            sns.boxplot(data=input_df, x=group_sub, y=column)
            plt.show()
        return out

    @staticmethod
    def get_shapiro_widgets(sub1, sub2):
        sub1_shapiro = stats.shapiro(sub1)
        sub2_shapiro = stats.shapiro(sub2)
        sub1_shapiro = widgets.HTML(
            value='statistics: {0}, pvalue: {1}'.format(round(sub1_shapiro.statistic, 4),
                                                        round(sub1_shapiro.pvalue, 4)),
            description="sub1's shapiro > ",
            disabled=True
        )
        sub2_shapiro = widgets.HTML(
            value='statistics: {0}, pvalue: {1}'.format(round(sub2_shapiro.statistic, 4),
                                                        round(sub2_shapiro.pvalue, 4)),
            description="sub2's shapiro > ",
            disabled=True
        )
        sharpiro_agg_widget = widgets.VBox([sub1_shapiro, sub2_shapiro])
        return sharpiro_agg_widget

    @staticmethod
    def get_int_test_ind_widget(sub1, sub2):
        res = stats.ttest_ind(sub1, sub2,
                              equal_var=True)
        sub1_int_test = widgets.HTML(
            value='statistics: {0}, pvalue: {1}'.format(
                round(res.statistic, 4), round(res.pvalue, 4)),
            description="inttest_ind > ",
            disabled=True
        )
        int_test_agg_widget = widgets.VBox([sub1_int_test])

        return int_test_agg_widget



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) kyoungjun.
# Distributed under the terms of the Modified BSD License.
import numpy as np
import pandas as pd
import statsmodels.api as sm

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_original import StatisticOperation
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget
from jupyter_analysis_extension.statistics_widget.regression.general_linear_regression import \
    WidgetGeneralLinearRegression


# from sklearn.linear_model import LinearRegression
# Graph


class GeneralLinearRegression(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        # Additional init codes for specific operations.
        self.op_name = "Linear Regression"
        self.sub = [
            {"name": "Dependent Variable", "type": ["N"], "max_count": 1, "min_count": 1},
            {"name": "Numeric Variables", "type": ["N"], "max_count": 0, "min_count": 0},
            {"name": "Categorical Variables", "type": ["C"], "max_count": 0, "min_count": 0}
        ]
        self.option = [
            {"name": "Baseline (CV)",
             "type": "individual_connected_unique_dropdown",
             "connect": ["Categorical Variables"], "acc_title": "List of CVs"}
        ]
        self.callback = {"refresh": refresh_tab}

        super(GeneralLinearRegression, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        """
        Construct the overall visual frames of linear regressions.
        Returns
        -------
        None
        """
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )
        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        """
        Perform the check of column types and the creation of analysis results of accordion widgets.
        Returns
        -------
        None
        """
        self.set_analyze_text('In progress...')

        # catch invalid data
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        dependent_var = None
        numeric_vars = None
        categorical_vars = None
        if 'Dependent Variable' in self.control_data.keys():
            dependent_var = self.column_value[self.control_data['Dependent Variable'][0]]
        if 'Numeric Variables' in self.control_data.keys():
            numeric_vars = []
            for item in self.control_data['Numeric Variables']:
                numeric_vars.append(self.column_value[item])
        if 'Categorical Variables' in self.control_data.keys():
            categorical_vars = []
            for item in self.control_data['Categorical Variables']:
                categorical_vars.append(self.column_value[item])

        option = self.control_data['option']
        baseline_cv = option['Baseline (CV)']

        result_widgets = self.get_result_widgets(dependent_var, numeric_vars, categorical_vars, baseline_cv)

        CommonUtil.update_result_layout(self.control_panel, result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):
        """
        Check the requirement numbers of columns and types.
        Returns
        -------
        str
            None if successful, error message otherwise.
        """
        error_msg = CommonUtil.check_variables_shapes(control_data, 'Dependent Variable', 'Number',
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Numeric Variables', 'Number',
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Categorical Variables',
                                                      ['N-Category', '2-Category'],
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        if ('Numeric Variables' not in control_data.keys()) and ('Categorical Variables' not in control_data.keys()):
            self.set_analyze_text('Please enter at least 1 column in Numeric Variables or Categorical Variables',
                                  'warning')
            return False

        return True

    def get_result_widgets(self, dependent_name, numeric_names, categorical_names, baseline_cv):
        if not categorical_names:
            eda_plot, stat_result_table, EDA_plot_title = self.get_linear_regression_info(dependent_name, numeric_names)
        else:
            eda_plot, stat_result_table, EDA_plot_title = self.get_linear_regression_with_categorical_info(
                dependent_name, numeric_names, categorical_names, baseline_cv)

        return CommonWidget.get_accordion_widget([eda_plot, stat_result_table],
                                                 [EDA_plot_title, 'Regression Model Info'])

    def get_linear_regression_info(self, dependent_name, numeric_names):
        df = self.input_df
        target_col_list = list(set([dependent_name] + numeric_names))
        # Drop unrelated columns
        df = df[target_col_list]
        # Drop the rows that have missing values in target columns.
        df = df.dropna(subset=target_col_list)

        x_data = np.array(df[numeric_names])
        # x_data_refined = x_data.reshape((-1, 1))
        y_data = np.array(df[dependent_name])

        # OLS Regression result
        x_data_sm = sm.add_constant(x_data)
        ls = sm.OLS(y_data, x_data_sm).fit()
        summary_table = ls.summary(yname=dependent_name, xname=['Intercept'] + numeric_names)
        # summary_table = ls.summary()
        # model = LinearRegression().fit(x_data, y_data)
        # Coefficient of determination, R^2
        # r_sq = round(model.score(x_data, y_data), 4)

        # coefficient (b_0)
        # intercept = np.round(model.intercept_, 4)
        # coefficient (b_1)
        # slope = np.round(model.coef_, 4)

        # Widget Generation
        # - EDA
        # Dependent & Target 들의 pairplot
        # Dependent & Target 들의 corr의 heatmap
        # (cate) Dependent & Target 들의 pairplot w/ categorical
        # eda_graphs = WidgetGeneralLinearRegression.get_eda_graphs()
        EDA_plot_title = "Exploratory Data Analysis (EDA)"
        sampled_df = df.copy()
        if len(self.input_df) > 10000:
            EDA_plot_title += " (Sampling 10000 rows)"
            # Sampling only 10000 rows
            sampled_df = sampled_df.sample(n=10000, random_state=1, replace=False)
        eda_plot = WidgetGeneralLinearRegression.get_eda_plot(sampled_df, target_col_list)

        # - Regression
        # R^2, intercept, ...
        # (cate x) regplotget_eda_plot
        # (cate) boxplot of Dependent & Target based on categorical
        # Coef
        # stat_result_table = WidgetGeneralLinearRegression.get_statistic_result_table(numeric_names, r_sq, intercept, slope)
        overall_results_as_html = summary_table.tables[0].as_html()
        params_results_as_html = summary_table.tables[1].as_html()
        # pd.read_html(results_as_html, header=0, index_col=0)[0]
        stat_result_table = WidgetGeneralLinearRegression.get_statistic_result_table(overall_results_as_html,
                                                                                     params_results_as_html)

        # Prediction
        # y_preds = model.predict(x_data)
        # print(y_preds.shape)
        # for numeric_name, y_pred in zip(numeric_names, y_preds):
        # sns.scatterplot(data=df, x=dependent_name, y=numeric_name)
        # sns.lineplot(x=x_data, y=y_pred)
        # plt.scatter(x_data_refined, y_data_refined, color='black')
        # plt.plot(x_data_refined, y_pred, color='blue', linewidth=3)

        return eda_plot, stat_result_table, EDA_plot_title

    def get_linear_regression_with_categorical_info(self, dependent_name, numeric_names, categorical_names,
                                                    baseline_cv):
        df = self.input_df
        type_list = self.column_type_list
        target_col_list = list(set([dependent_name] + numeric_names + categorical_names))

        # Drop unrelated columns
        df = df[target_col_list]

        # Drop the rows that have missing values in target columns.
        df = df.dropna(subset=target_col_list)
        # consider k-1 levels of k categorical levels if drop_first is True.
        twocategory = []
        set_baseline = []
        multicategory = []
        drop_baseline = []
        for key, value in baseline_cv.items():
            if type_list[key] == '2-Category':
                uniq = df[key].dropna().unique()
                add = str(uniq[uniq != value][0])
                df[key + '_' + add] = (df[key] != value).astype(int)
                drop_baseline.append(key)
            else:
                multicategory.append(key)
                drop_baseline.append(key + '_' + str(value))
        if len(multicategory) > 0:
            typed_df = df[multicategory].astype(str)
            df_dummy = pd.concat((df, pd.get_dummies(typed_df, prefix=multicategory, drop_first=False)),
                                 axis=1)
        else:
            df_dummy = df
        df_dummy = df_dummy.drop(multicategory + drop_baseline, axis=1)
        x_cols_dummy = list(df_dummy.columns)
        x_cols_dummy.remove(dependent_name)
        x_data = np.array(df_dummy[x_cols_dummy])
        # x_data_refined = x_data.reshape((-1, 1))
        y_data = np.array(df_dummy[dependent_name])

        # OLS Regression result
        x_data_sm = sm.add_constant(x_data)
        ls = sm.OLS(y_data, x_data_sm).fit()
        summary_table = ls.summary(yname=dependent_name, xname=['Constant'] + x_cols_dummy)

        # Widget Generation
        # - EDA
        # Dependent & Target 들의 pairplot
        # Dependent & Target 들의 corr의 heatmap
        # (cate) Dependent & Target 들의 pairplot w/ categorical
        # eda_graphs = WidgetGeneralLinearRegression.get_eda_graphs()
        EDA_plot_title = "Exploratory Data Analysis (EDA)"
        sampled_df = df.copy()
        if len(self.input_df) > 10000:
            EDA_plot_title += " (Sampling 10000 rows)"
            # Sampling only 10000 rows
            sampled_df = sampled_df.sample(n=10000, random_state=1, replace=False)
        eda_plot = WidgetGeneralLinearRegression.get_categorical_eda_plot(sampled_df, [dependent_name] + numeric_names,
                                                                          categorical_names)

        # - Regression
        # R^2, intercept, ...
        # (cate x) regplot
        # (cate) boxplot of Dependent & Target based on categorical
        # Coef
        # stat_result_table = WidgetGeneralLinearRegression.get_statistic_result_table(numeric_names, r_sq, intercept, slope)
        overall_results_as_html = summary_table.tables[0].as_html()
        params_results_as_html = summary_table.tables[1].as_html()
        # pd.read_html(results_as_html, header=0, index_col=0)[0]
        stat_result_table = WidgetGeneralLinearRegression.get_statistic_result_table(overall_results_as_html,
                                                                                     params_results_as_html)

        # Prediction
        # y_preds = model.predict(x_data)
        # print(y_preds.shape)
        # for numeric_name, y_pred in zip(numeric_names, y_preds):
        # sns.scatterplot(data=df, x=dependent_name, y=numeric_name)
        # sns.lineplot(x=x_data, y=y_pred)
        # plt.scatter(x_data_refined, y_data_refined, color='black')
        # plt.plot(x_data_refined, y_pred, color='blue', linewidth=3)

        return eda_plot, stat_result_table, EDA_plot_title



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) TmaxBI.
# Distributed under the terms of the Modified BSD License.

from statsmodels.stats.anova import AnovaRM

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_original import StatisticOperation
from jupyter_analysis_extension.statistics_widget.anova.anova_rm import WidgetANOVARM
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget


class ANOVARM(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        self.op_name = "Repeated Measure ANOVA"
        self.sub = [
                       {"name": "Dependent Variable (R.M.)", "type": ["N"], "max_count": 1, "min_count": 1},
                       {"name": "Subject ID", "type": [], "max_count": 1, "min_count": 1},
                       {"name": "Within-Subject Factrs", "type": ["C"], "max_count": 0, "min_count": 0}
                   ]
        self.option = []
        self.callback = {"refresh": refresh_tab}

        super(ANOVARM, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )

        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        self.set_analyze_text('In progress...')

        # catch invalid data
        option = self.control_data['option']
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        # get analysis data & option
        degree_freedom = len(self.input_df) - 2

        dependent_var = list(self.control_data['Dependent Variable (R.M.)'])[0]

        # self.input_df.dropna(inplace = True)

        subject_id = list(self.control_data['Subject ID'])[0]
        if 'Within-Subject Factors' in self.control_data.keys():
            within_factors_list = list(self.control_data['Within-Subject Factors'])
        else:
            within_factors_list = []
        result_widgets = self.get_result_widgets(
            degree_freedom, dependent_var, subject_id, within_factors_list)
        if CommonUtil.update_result_layout(self.control_panel, result_widgets):
            self.set_analyze_text('Done.')
        else:
            return

    def validate_input_data(self, control_data):

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Dependent Variable (R.M.)', 'Number',
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Subject ID', ['Object', 'N-Category', 'Number'],
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Within-Subject Factors',
                                                      ['N-Category', '2-Category'],
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        return True

    def get_result_widgets(self, degree_freedom, dependent_var, subject_id, within_factors_list):
        statistic_chart = []
        statistic_chart_title = []
        dep_var = self.column_value[dependent_var]
        subject = self.column_value[subject_id]
        within_factors_ls = []
        if len(within_factors_list) > 0:
            for within_factor in within_factors_list:
                within_factors_ls.append(self.column_value[within_factor])
        try:
            aovrm = AnovaRM(self.input_df, dep_var, subject, within=within_factors_ls)
            table = aovrm.fit().anova_table
        except Exception as e:
            self.set_analyze_text(str(e).split('.')[0], 'warning')
            return None

        statistic_result_table = WidgetANOVARM.get_statistic_result_table(table)

        """
        boxplot_sns = WidgetANOVAANCOVA.get_boxplot_sns()
        statistic_chart.append(boxplot_sns)
        statistic_chart_title.append('Box Plot (plotly): ' + str(dep))
        """

        return CommonWidget.get_accordion_widget([statistic_result_table, *statistic_chart],
                                                 ['Test Result(s)', *statistic_chart_title])



#!/usr/bin/env python
# coding: utf-8

import numpy as np
# Copyright (c) kyoungjun.
# Distributed under the terms of the Modified BSD License.
from scipy import stats

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_original import StatisticOperation
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget
from jupyter_analysis_extension.statistics_widget.t_test.t_test_pair_sample import WidgetTTestPairSample


class TTestPairSample(StatisticOperation):

    def __init__(self, df, column_type_list, refresh_tab, logger):
        self.op_name = "T-Test : Paired Sample"
        self.sub = [
                  {"name": "Pair Set 1", "type": ["N"], "max_count": 0, "min_count": 1},
                  {"name": "Pair Set 2", "type": ["N"], "max_count": 0, "min_count": 1}
              ]
        self.option = [
                     {"name": "Test Types", "type": "checkbox", "value": ["Student\'s", "Wilcoxon signed-rank"]},
                     {"name": "Hypothesis", "type": "radio",
                      "value": ["Measure 1 ≠ Measure 2", "Measure 1 > Measure 2", "Measure 1 < Measure 2"]}
                 ]
        self.callback = {"refresh": refresh_tab}

        super(TTestPairSample, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )
        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        self.set_analyze_text('In progress...')

        # catch invalid data
        option = self.control_data['option']
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        # get analysis data & option
        degree_freedom = len(self.input_df)-1

        pair_set_1_layout_var_list = list(self.control_data['Pair Set 1'])
        pair_set_1_var_list = []
        for layout_var in pair_set_1_layout_var_list:
            pair_set_1_var_list.append(self.column_value[layout_var])

        pair_set_2_layout_var_list = list(self.control_data['Pair Set 2'])
        pair_set_2_var_list = []
        for layout_var in pair_set_2_layout_var_list:
            pair_set_2_var_list.append(self.column_value[layout_var])

        tests_students = option['Test Types']['Student\'s']
        tests_wilcoxon = option['Test Types']['Wilcoxon signed-rank']

        hypothesis_alternative = option['Hypothesis']['radio']
        if hypothesis_alternative == 'Measure 1 ≠ Measure 2':
            hypothesis_alternative = 'two-sided'
        elif hypothesis_alternative == 'Measure 1 > Measure 2':
            hypothesis_alternative = 'greater'
        elif hypothesis_alternative == 'Measure 1 < Measure 2':
            hypothesis_alternative = 'less'

        result_widgets = self.get_result_widgets(
            degree_freedom, pair_set_1_var_list, pair_set_2_var_list, [tests_students, tests_wilcoxon], [hypothesis_alternative])

        CommonUtil.update_result_layout(self.control_panel, result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):
        error_msg = CommonUtil.check_option_checkbox(control_data, 'Test Types', 1)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Pair Set 1', 'Number',
                                                     self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Pair Set 2', 'Number',
                                                     self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        pair_set_1_var_list = list(control_data['Pair Set 1'])
        pair_set_2_var_list = list(control_data['Pair Set 2'])

        if len(pair_set_1_var_list) != len(pair_set_2_var_list):
            self.set_analyze_text(
                'Please make Pair Set 1,2 equal in length.', "warning")
            return False

        if (np.array(pair_set_1_var_list) == np.array(pair_set_2_var_list)).max():
            self.set_analyze_text(
                'Only different columns can be compared.', "warning")
            return False

        return True

    def get_result_widgets(self, degree_freedom, pair_set_1_var_list, pair_set_2_var_list, tests_options, hypothesis_options):
        target_column_names = []
        statistic_tests_options = []
        statistic_chart = []
        statistic_result = []

        for index, item in enumerate(pair_set_1_var_list):
            target_column_names.append(
                pair_set_1_var_list[index] + ' | ' + pair_set_2_var_list[index])
            statistic_chart.append(self.input_df[item])
            statistic_chart.append(self.input_df[pair_set_2_var_list[index]])
            if tests_options[0] == True:
                statistic_result.append(stats.ttest_rel(
                    self.input_df[item], self.input_df[pair_set_2_var_list[index]], nan_policy='omit', alternative=hypothesis_options[0]))
                statistic_tests_options.append('Student\'s')

            if tests_options[1] == True:
                if tests_options[0] == True:
                    target_column_names.append('')
                statistic_result.append(stats.wilcoxon(
                    self.input_df[item], self.input_df[pair_set_2_var_list[index]], zero_method="wilcox", alternative=hypothesis_options[0]))
                statistic_tests_options.append('Wilcoxon signed-rank')

        statistic_result_table = WidgetTTestPairSample.get_statistic_result_table(
            target_column_names, degree_freedom, statistic_result, statistic_tests_options)

        box_plot_title = "Box Plot"
        if len(self.input_df) > 10000:
            # Sampling only 10000 rows
            box_plot_title += " (Sampling 10000 rows)"
            for index, df in enumerate(statistic_chart):
                statistic_chart[index] = df.copy().sample(n=10000, random_state=1, replace=False)

        box_plot_plotly = WidgetTTestPairSample.get_box_plot_plotly(
            statistic_chart)

        return CommonWidget.get_accordion_widget([statistic_result_table, box_plot_plotly], ['Test Result(s)', box_plot_title])




import ipywidgets as widgets

# Graph
import plotly.graph_objs as go


class WidgetBasicAnalysis:
    @ staticmethod
    def get_statistic_result_table(name, degree_freedom, test_result, statistic_tests_options):
        res_table_header = ''
        variable_list = list(test_result["Base(기초)"].keys())
        variable_count = len(test_result["Base(기초)"].items())

        for key, value in test_result["Base(기초)"].items():
            res_table_header += """<th>{0}</th>""".format(key)
            res_table_row_info =""
            for key_1, value_1 in value.items():
                res_table_row_info_child =""
                for i in range(0, variable_count):
                    res_table_row_info_child += """
                    <td>{0}</td>
                    """.format(test_result["Base(기초)"][variable_list[i]][key_1])
                res_table_row_info += """
                                    <tr>
                                        <td>{0}</td>
                                        {1}
                                    </tr>
                                """.format(key_1,res_table_row_info_child)

        res_table = widgets.HTML(
            value="""
                <table style="width:100%">
                <tr style="border-bottom: 1px solid black;">
                    <th></th>
                    {0}
                </tr>
               {1}
                </table>""".format(res_table_header, res_table_row_info),
            disabled=True
        )
        statistic_result_table = widgets.VBox([res_table])
        statistic_result_table.add_class('styledTable')

        return statistic_result_table

    @staticmethod
    def get_box_plot_plotly(data_list):
        fig = go.FigureWidget()
        for data in data_list:
            fig.add_trace(
                go.Box(y=data,
                       name=data.name,
                       boxpoints='all',
                       jitter=0.2,
                       pointpos=-1.8
                       ))

        fig.update_layout(
            width=500,
            height=500,
        )
        return fig



import pandas as pd
from scipy.stats import binom_test

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_original import StatisticOperation
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget
from jupyter_analysis_extension.utils.widget_maker import WidgetMaker


class FrequenciesProportion2(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        self.op_name = "Proportion (2-Outcomes)"
        self.sub = [
            {"name": "Variable(s)", "type": ["N"], "max_count": 0, "min_count": 1}
        ]
        self.option = [
            {"name": "Hypothesis", "type": "numberText-radio", "value": {
                "numberText": {"name": "Test value", "value": 0.5},
                "radio": {"name": None, "value": ["≠ Test value", "> Test value", "< Test value"]}
            }}
        ]
        self.callback = {"refresh": refresh_tab}
        super(FrequenciesProportion2, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )

        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        self.set_analyze_text('In progress...')

        # catch invalid data
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        # get analysis data & option
        var_layout_var_list = list(self.control_data['Variable(s)'])
        var_var_list = []
        for layout_var in var_layout_var_list:
            var_var_list.append(self.column_value[layout_var])

        option = self.control_data['option']
        hypothesis_test_value = option['Hypothesis']['number']
        hypothesis_alternative = option['Hypothesis']['radio']
        if hypothesis_alternative == '≠ Test value':
            hypothesis_alternative = 'two-sided'
        elif hypothesis_alternative == '> Test value':
            hypothesis_alternative = 'greater'
        elif hypothesis_alternative == '< Test value':
            hypothesis_alternative = 'less'

        result_widgets = self.get_result_widgets(var_var_list, hypothesis_test_value, hypothesis_alternative)

        CommonUtil.update_result_layout(self.control_panel, result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):
        error_msg = CommonUtil.check_variables_shapes(control_data, 'Variable(s)',
                                                      ['Object', "Number", "N-category", '2-Category'],
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        return True

    def get_result_widgets(self, var_var_list, hypothesis_test_value, hypothesis_alternative):
        header_list = [" ", "Item", "Count", "Total", "Proportion", "p-value"]

        footer_test_value_equal_mark = ""
        if hypothesis_alternative == 'two-sided':
            footer_test_value_equal_mark = '≠'
        elif hypothesis_alternative == 'greater':
            footer_test_value_equal_mark = '>'
        elif hypothesis_alternative == 'less':
            footer_test_value_equal_mark = '<'

        footer_text = """Hypothesis Proportion {0} {1}""".format(footer_test_value_equal_mark, hypothesis_test_value)

        body_list = []
        for target_column in var_var_list:
            proportion_count_list = self.input_df[target_column].groupby(self.input_df[target_column]).count()
            proportion_count_key_list = proportion_count_list.keys()
            proportion_count_value_list = proportion_count_list.values
            proportion_count_value_sum = sum(proportion_count_value_list)

            for index, item in enumerate(proportion_count_value_list):
                target_column_name = target_column if index == 0 else ""

                p_value = binom_test(item, n=proportion_count_value_sum, p=hypothesis_test_value,
                                     alternative=hypothesis_alternative)
                proportion = round(item / proportion_count_value_sum, 3)
                body_list.append(
                    [target_column_name, proportion_count_key_list[index], item, proportion_count_value_sum, proportion,
                     round(p_value, 3)])

        data_table = WidgetMaker.get_styled_table(header_list, body_list, footer_text)

        return CommonWidget.get_accordion_widget([data_table], ['Data table'])



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) TmaxBI.
# Distributed under the terms of the Modified BSD License.

import statsmodels.api as sm
from statsmodels.formula.api import ols
from scipy.stats import friedmanchisquare

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_op import StatisticOperation
from jupyter_analysis_extension.statistics_widget.anova.anova_ancova import WidgetANOVAANCOVA
from jupyter_analysis_extension.statistics_widget.anova.anova_friedman import WidgetANOVAFriedman
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget


class ANOVAFriedman(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        # Data
        self.input_df = df
        self.column_type_list = column_type_list
        self.op_name = "Repeated Measure(Friedman)"
        self.sub = [
            {"name": "Measures", "type": ["N"], "max_count": 0, "min_count": 1},
        ]
        self.option = []
        self.callback = {"refresh": refresh_tab}

        super(ANOVAFriedman, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )

        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        self.set_analyze_text('In progress...')

        # catch invalid data
        option = self.control_data['option']
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        # get analysis data & option
        measures_list = []
        for measures in self.control_data['Measures']:
            measures_list.append(self.column_value[measures])

        degree_freedom = len(measures_list) - 1

        result_widgets = self.get_result_widgets(degree_freedom, measures_list)

        CommonUtil.update_result_layout(self.control_panel, result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Measures', 'Number',
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        return True

    def get_result_widgets(self, degree_freedom, measures_list):

        statistic_chart = []
        statistic_chart_title = []

        friedmanchisquare_parameter = []
        for measures in measures_list:
            friedmanchisquare_parameter.append(self.input_df[measures])
        print(friedmanchisquare_parameter)
        statistic_result = friedmanchisquare(*friedmanchisquare_parameter)

        statistic_result_table = WidgetANOVAFriedman.get_statistic_result_table(degree_freedom, statistic_result)

        return CommonWidget.get_accordion_widget([statistic_result_table, ], ['Test Result(s)'])



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) kyoungjun.
# Distributed under the terms of the Modified BSD License.
import ipywidgets as widgets
import matplotlib.pyplot as plt
import plotly.graph_objs as go


class WidgetPartialCorrelation:

    @staticmethod
    def get_x_dropdown(columns):
        return widgets.Dropdown(
            options=columns,
            description='X Variable: ',
            style=dict(description_width='initial')
        )

    @staticmethod
    def get_y_dropdown(columns):
        return widgets.Dropdown(
            options=columns,
            description='Y Variable: ',
            style=dict(description_width='initial')
        )

    @staticmethod
    def get_click_button():
        return widgets.Button(
            description='Analyze',
            disabled=False,
            button_style='',
            tooltip='Analyze',
            icon='check')

    @staticmethod
    def get_agg_widget(x_dropdown, y_dropdown, click_button):
        margin = widgets.Layout(margin='5px 5px 5px 5px', align_items='center')
        agg_widget = widgets.VBox([widgets.HBox([x_dropdown, y_dropdown, click_button], layout=margin)], layout=margin)
        return agg_widget

    @staticmethod
    def get_accordion_widgets(r_sq, intercept, slope, plt):
        description_widget = WidgetPartialCorrelation.get_description_widget(r_sq, intercept, slope)
        graph_widget = WidgetPartialCorrelation.get_graph_widget(plt)

        accordion_widget = widgets.Accordion(children=[description_widget, graph_widget]
                                             , layout=widgets.Layout(width='100%'))
        title_list = ['Description', 'Graph Explanation']
        for i in range(len(accordion_widget.children)):
            accordion_widget.set_title(i, title_list[i])

        return accordion_widget

    @staticmethod
    def get_description_widget(r_sq, intercept, slope):
        r_sq_widget = widgets.HTML(
            value='R^2 Square: {0}'.format(r_sq),
            disabled=True
        )
        intercept_widget = widgets.HTML(
            value='Intercept: {0}'.format(4),
            disabled=True
        )
        slope_widget = widgets.HTML(
            value='Slope: {0}'.format(slope),
            disabled=True
        )
        description_widget = widgets.VBox([r_sq_widget, intercept_widget, slope_widget])
        return description_widget

    @staticmethod
    def get_graph_widget():
        out = widgets.Output()
        with out:
            plt.show()
        return out

    @staticmethod
    def get_statistic_sep_result_table(col_names, r_sq, intercept, slope):
        model_info_table = widgets.HTML(
            value="""
                <table style="width:100%">
                <tr style="border-bottom: 1px solid black;">
                    <th>R^2 Square</th>
                    <th>Intercept</th>
                </tr>
                        <tr>
                            <td>{0}</td>
                            <td>{1}</td>
                        </tr>
                </table>""".format(r_sq, intercept, slope),
            disabled=True
        )
        res_table_info = ''
        for idx, name in enumerate(col_names):
            res_table_info += """
                        <tr>
                            <td>{0}</td>
                            <td>{1}</td>
                        </tr>
                        """.format(name, slope[idx])

        res_table = widgets.HTML(
            value="""
                <table style="width:100%">
                <tr style="border-bottom: 1px solid black;">
                    <th>Target Column</th>
                    <th>Slope</th>
                </tr>
               {0}
                </table>""".format(res_table_info),
            disabled=True
        )
        statistic_result_table = widgets.VBox([model_info_table, res_table])
        statistic_result_table.add_class('styledTable')

        return statistic_result_table

    @staticmethod
    def get_statistic_result_table(corr_info, corr_coefficients, addi_ops):
        """
        Parameters
        ----------
        corr_info : dict
        corr_coefficients : list
        addi_ops : list

        Returns
        -------
        object
        """
        res_table_info = ''

        for col_names in corr_info.keys():
            pearson_data = ""
            spearman_data = ""
            if "Report significance (p-value)" in addi_ops:
                if "Pearson" in corr_coefficients:
                    pearson_data = "{0}<br>(p-value:{1}, n: {2}, CI 95%:{3})".format(
                        corr_info[col_names]["Pearson(r)"],
                        corr_info[col_names]["Pearson(p)"],
                        corr_info[col_names]["Pearson(n)"],
                        corr_info[col_names][
                            "Pearson(CI95)"])
                if "Spearman" in corr_coefficients:
                    spearman_data = "{0}<br>(p-value:{1}, n: {2}, CI 95%:{3})".format(
                        corr_info[col_names]["Spearman(r)"],
                        corr_info[col_names]["Spearman(p)"],
                        corr_info[col_names]["Spearman(n)"],
                        corr_info[col_names]["Spearman(CI95)"])
            else:
                if "Pearson" in corr_coefficients:
                    pearson_data = corr_info[col_names]["Pearson(r)"]
                if "Spearman" in corr_coefficients:
                    spearman_data = corr_info[col_names]["Spearman(r)"]

            if pearson_data != "":
                pearson_data = "<td>{0}</td>".format(pearson_data)
            if spearman_data != "":
                spearman_data = "<td>{0}</td>".format(spearman_data)

            res_table_info += """
                        <tr>
                            <td>{0}</td>
                            <td>{1}</td>
                            {2}
                            {3}
                        </tr>
                        """.format(col_names[0], col_names[1], pearson_data, spearman_data)

        res_table = widgets.HTML(
            value="""
                <table style="width:100%">
                <tr style="border-bottom: 1px solid black;">
                    <th colspan="2">Target Columns</th>
                    {0}
                    {1}
                </tr>
               {2}
                </table>""".format("<th>Pearson</th>" if "Pearson" in corr_coefficients else "",
                                   "<th>Spearman</th>" if "Spearman" in corr_coefficients else "",
                                   res_table_info),
            disabled=True
        )

        statistic_result_table = widgets.VBox([res_table])
        statistic_result_table.add_class('styledTable')
        return statistic_result_table

    @staticmethod
    def get_all_scatter_plot(data, target_col, control_col):
        dim_list = []
        for column in target_col + control_col:
            dim_dict = {'label': column, 'values': data[column]}
            dim_list.append(dim_dict)
        fig = go.FigureWidget()
        # fig.add_trace(go.Splom(dimensions=[dict(label='Period', values=data['Period']), dict(label='Data_value', values=data['Data_value'])]))
        fig.add_trace(go.Splom(dimensions=dim_list))
        fig.update_layout(
            title='Scatter Plot',
            title_x=0.5,
            title_font_size=30,
            dragmode='select',
            width=600,
            height=600,
            hovermode='closest',
        )

        return fig

    @staticmethod
    def get_heatmap_plot(data, columns):
        fig = go.FigureWidget()
        fig.add_trace(go.Heatmap(x=columns, y=columns, z=data.corr()))
        fig.update_layout(
            title='Heatmap of Variables',
            title_x=0.5,
            title_font_size=30,
            dragmode='select',
            width=600,
            height=600,
            hovermode='closest',
        )
        return fig

    @staticmethod
    def get_all_scatter_categorical_plot(data, columns, categorical_names):
        dim_list = []

        for column in columns:
            dim_dict = {'label': column, 'values': data[column]}
            dim_list.append(dim_dict)
        fig = go.FigureWidget()
        for name in categorical_names:
            index_vals = data[name].astype('category').cat.codes
            # fig.add_trace(go.Splom(dimensions=[dict(label='Period', values=data['Period']), dict(label='Data_value', values=data['Data_value'])]))
            fig.add_trace(go.Splom(dimensions=dim_list, text=data[name], textsrc="aa",
                                   marker=dict(color=index_vals, showscale=True,
                                               line_color='white', line_width=0.5)))
        fig.update_layout(
            title='Scatter Plot',
            title_x=0.5,
            title_font_size=30,
            dragmode='select',
            width=600,
            height=600,
            hovermode='closest',
        )

        return fig

    @staticmethod
    def get_eda_plot(data, columns):
        scatter_plot = WidgetPartialCorrelation.get_all_scatter_plot(data, columns)
        heatmap_plot = WidgetPartialCorrelation.get_heatmap_plot(data, columns)
        return widgets.HBox([scatter_plot, heatmap_plot])

    @staticmethod
    def get_categorical_eda_plot(data, columns, categorical_names):
        pair_plot = WidgetPartialCorrelation.get_all_scatter_categorical_plot(data, columns, categorical_names)
        return widgets.HBox([pair_plot])



from abc import *

class StatisticOperation:
    """operation default class"""
    # main integrated widget
    control_panel = None

    def __init__(self, op_name, df, column_type_list, refresh_tab, logger):
        # Data
        self.input_df = df
        self.column_type_list = column_type_list
        self.op_name = op_name

        self.column_value = {}
        self.control_data = None

        # Widget & Layout
        self.control_panel = None
        self.analyze_layout = None
        self.analyze_textfield = None
        self.refresh_textfield = None

        # func
        self.refresh_tab = refresh_tab

        # log
        self.logger = logger

        self.construct_control_panel()
    @abstractmethod
    def construct_control_panel(self):
        pass

    def refresh_tab(self):
        if self.analyze_textfield is not None:
            self.analyze_textfield.value=""

        if self.refresh_textfield is not None:
            self.refresh_textfield.value=""
        return

    def get_tab_widget(self):
        return self.control_panel


    def connect_button_layout(self, button_layout, analyze_btn_event):
        """

        Parameters
        ----------
        button_layout
            button-textField widget layout
        analyze_btn_event
            trigger analyze btn event
        -------
        """
        self.button_layout = button_layout
        self.analyze_textfield = button_layout.children[0].children[0]
        self.refresh_textfield = button_layout.children[0].children[1]
        self.button_layout.children[1].children[0].on_click(analyze_btn_event)

    def set_analyze_text(self, text, mode="plain"):
        """

        Parameters
        ----------
        text
            real textField string
        mode
            plain | warning
        """
        target_text = """<div style="display:flex;justify-content:flex-end">{0}</div>""".format(
            text)
        if mode == "warning":
            target_text = """<div style="font-size:1.5rem;color:red;display:flex;justify-content:flex-end">{0}</div>""".format(
                text)
        self.analyze_textfield.value = target_text
    
    def set_refresh_text(self, text):
        """

        Parameters
        ----------
        text
            real textField string
        """
        if hasattr(self, "refresh_textfield"):
            self.refresh_textfield.value = """<div style="font-size:1.5rem;color:red;display:flex;justify-content:flex-end">{0}</div>""".format(
                text)


import ipywidgets as widgets

# Graph
import plotly.graph_objs as go


class WidgetFrequenciesChiSquare:
    @staticmethod
    def get_statistic_result_table(first_col_values, df_count, statistic_result, statistic_tests_options):
        res_table_info = ''

        for key in statistic_result.keys():
            if key == 'Chi-square':
                res_table_info += """
                                    <tr>
                                        <td>{0}</td>
                                        <td>{1}</td>
                                        <td>{2}</td>
                                        <td>{3}</td>
                                    </tr>
                                    """.format(key, round(statistic_result[key][0], 1), statistic_result[key][2], round(statistic_result[key][1], 3))
            if key == "Fisher\'s exact":
                res_table_info += """
                                    <tr>
                                        <td>{0}</td>
                                        <td>{1}</td>
                                        <td>{2}</td>
                                        <td>{3}</td>
                                    </tr>
                                    """.format(key, "-", "-", round(statistic_result[key][0], 3))

        res_table_info += """
                    <tr>
                        <td>{0}</td>
                        <td>{1}</td>
                        <td>{2}</td>
                        <td>{3}</td>
                    </tr>
                    """.format("N", df_count, "", "")

        res_table = widgets.HTML(
            value="""
                <table>
                <tr>
                    <th></th>
                    <th>Value</th>
                    <th>df</th>
                    <th>p-value</th>
                </tr>
               {0}
                </table>""".format(res_table_info),
            disabled=True
        )
        statistic_result_table = widgets.VBox([res_table])
        statistic_result_table.add_class('styledTable')

        return statistic_result_table

    @staticmethod
    def get_box_plot_plotly(data_list):
        fig = go.FigureWidget()
        for data in data_list:
            fig.add_trace(
                go.Box(y=data, name=data.name))
        return fig

    @staticmethod
    def get_data_table(cross_tab):
        tab_column_length = len(cross_tab.to_numpy()[0]) + 1
        res_header_info = "<th></th>"
        for index, item in enumerate(cross_tab.columns):
            res_header_info += "<th>{0}</th>".format(item)

        res_table_info = ""
        for index, item in enumerate(cross_tab.index):
            res_row_info = "<td>{0}</td>".format(item)
            for count in range(tab_column_length-1):
                res_row_info += """
                            <td>{0}</td>
                            """.format(cross_tab.to_numpy()[index][count])
            res_table_info += """
                        <tr>
                            {0}
                        </tr>
                        """.format(res_row_info)

        res_table = widgets.HTML(
            value="""
                <table table-layout: fixed;">
                <tr>
                    {0}
                </tr>
                {1}
                </table>""".format(res_header_info,res_table_info),
            disabled=True
        )
        res_table.add_class('styledTable')
        return widgets.VBox([res_table])



import pandas as pd
from scipy import stats

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_original import StatisticOperation
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget
from jupyter_analysis_extension.utils.widget_maker import WidgetMaker


class FrequenciesProportion(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        self.op_name = "Proportion (N-Outcomes)"
        self.sub = [
            {"name": "Variable", "type": ["N"], "max_count": 1, "min_count": 1},
        ]
        self.option = [
            {"name": "Expected", "type": "checkbox", "value": ["Visible"]},
            {"name": "Expected Proportion",
             "type": "individual_connected_unique_numberField",
             "connect": ["Variable"], "acc_title": "change proportion"}
        ]
        self.callback = {"refresh": refresh_tab}
        super(FrequenciesProportion, self).__init__(self.op_name, df, column_type_list, refresh_tab,
                                                    logger)

    def construct_control_panel(self):
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )

        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        self.set_analyze_text('In progress...')

        # catch invalid data
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        # get analysis data & option

        var_layout_var_list = list(self.control_data['Variable'])
        var_var_list = []
        for layout_var in var_layout_var_list:
            var_var_list.append(self.column_value[layout_var])

        option = self.control_data['option']
        option_expected_visible = option['Expected']['Visible']
        option_expected_proportion = option['Expected Proportion']

        result_widgets = self.get_result_widgets(var_var_list, option_expected_proportion, option_expected_visible)

        CommonUtil.update_result_layout(self.control_panel, result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):
        error_msg = CommonUtil.check_variables_shapes(control_data, 'Variable',
                                                      ['Object', "Number", "N-category", '2-Category'],
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        return True

    def get_result_widgets(self, var_var_list, option_expected_proportion, option_expected_visible):

        statistic_result = {}
        target_column = var_var_list[0]

        proportion_count_list = self.input_df[target_column].groupby(self.input_df[target_column]).count()
        proportion_count_key_list = proportion_count_list.keys()
        proportion_count_value_list = proportion_count_list.values
        proportion_count_value_sum = sum(proportion_count_value_list)

        expected_proportion_count_list = [1 for i in self.input_df[target_column].unique()]
        for key, value in option_expected_proportion.items():
            type_key = key if self.column_type_list[target_column] != 'Number' else int(key)
            expected_proportion_count_list[list(proportion_count_list.keys()).index(type_key)] = value
        expected_proportion_radio_sum = sum(expected_proportion_count_list)

        for index, item in enumerate(expected_proportion_count_list):
            expected_proportion_count_list[index] = round(
                item * proportion_count_value_sum / expected_proportion_radio_sum)

        expected_proportion_count_value_sum = sum(expected_proportion_count_list)
        origin_expected_count_0 = expected_proportion_count_list[0]
        difference_correction_value = proportion_count_value_sum - expected_proportion_count_value_sum
        if proportion_count_value_sum != expected_proportion_count_value_sum:
            expected_proportion_count_list[0] += difference_correction_value

        # result table
        df_count = len(self.input_df[target_column].unique()) - 1
        statistic_result["Proportion"] = stats.chisquare(list(proportion_count_list.values),
                                                         f_exp=expected_proportion_count_list)
        statistic_table_header = ["statistic", "df", "p-value"]
        statistic_table_body = [[round(statistic_result["Proportion"].statistic, 1), df_count,
                                 round(statistic_result["Proportion"].pvalue, 3)]]
        statistic_table = WidgetMaker.get_styled_table(statistic_table_header, statistic_table_body)

        # data table
        header_list = ["Item", "Count", "Proportion"]
        footer_text = ""
        if option_expected_visible:
            header_list = ["Item", " ", "Count", "Proportion"]
            footer_text = "*Values for correcting differences : [{0}] : {1} ({2})".format(
                list(proportion_count_key_list)[0],
                origin_expected_count_0,
                difference_correction_value)
        body_list = []
        for index, item in enumerate(proportion_count_list.keys()):
            if option_expected_visible:
                body_list.append([item, "observed", proportion_count_value_list[index],
                                  round(int(proportion_count_value_list[index]) / int(proportion_count_value_sum), 3)])
                body_list.append(["", "expected", expected_proportion_count_list[index],
                                  round(int(expected_proportion_count_list[index]) / int(proportion_count_value_sum),
                                        3)])
            else:
                body_list.append([item, proportion_count_value_list[index],
                                  round(int(proportion_count_value_list[index]) / int(proportion_count_value_sum), 3)])

        data_table = WidgetMaker.get_styled_table(header_list, body_list, footer_text)

        return CommonWidget.get_accordion_widget([statistic_table, data_table], ['Test Result(s)', 'Data table'])



import ipywidgets
from IPython.display import display, Javascript


class WidgetStyle:
    @staticmethod
    def style_default():
        style_data = ""
        style_data += WidgetStyle.style_notebook_windows()
        style_data += WidgetStyle.style_op_popup_menu()
        style_data += WidgetStyle.style_table()
        style_data += WidgetStyle.style_hyper()
        style_data += WidgetStyle.style_file_loading()
        style_data += WidgetStyle.style_check_box()

        integrated_style = ipywidgets.HTML(
            value="""
                <style>
                {0}
                </style
            """.format(style_data)
        )
        display(integrated_style)

    @staticmethod
    def style_notebook_windows():
        style_data = """
                            div#notebook-container    { width: 100%;}
                            div#menubar-container     { width: 65%; }
                            div#maintoolbar-container { width: 99%; }
                            div#output_scroll {height: 100%; }
        """
        disable_js = """
        IPython.OutputArea.prototype._should_scroll = function(lines) {
            return false;
        }
        """
        display(Javascript(disable_js))
        return style_data

    @staticmethod
    def style_op_popup_menu():
        style_data = """
                   .icon_menu {
                     overflow: visible !important;
                   }
                   .drop-img {
                     background-color: white;
                     color: white;
                     cursor: pointer;
                     border: none;
                     width: 100%; 
                     height: 100%;
                     padding : 15px;
                     position : relative;
                     top : -5px;
                   }
                   .icon-op-name {
                     position: relative;
                     text-align: center;
                     top: -24px;
                     font-size : 11px;
                     cursor: pointer;
                   }
                   .dropdown-btn {
                     width: 65px;
                     height:65px;
                     border : 1px solid grey;
                     border-radius: 14%;
                     overflow : hidden !important;
                   }
                   .dropdown {
                     width: 50px;
                     position: relative;
                     display: inline-block;
                     overflow: visible !important;
                   }
                   .dropdown-content {
                     display: none;
                     position: absolute;
                     background-color: white;
                     min-width: 200px;
                     box-shadow: 0px 8px 16px 0px rgba(0, 0, 0, 0.2);
                     z-index: 100;
                     cursor: pointer;
                   }
                   .dropdown-content a {
                     color: black;
                     padding: 10px;
                     text-decoration: none;
                     display: block;
                     min-width: 200px;
                     cursor: pointer;
                   }
                   .dropdown:hover .drop-img {
                     background-color: aliceblue;
                   }
                   .dropdown-content a:hover {
                     background-color: aliceblue;
                   }
                   .dropdown:hover .dropdown-content {
                     display: block;
                   }
                """
        return style_data

    @staticmethod
    def style_table():
        style_data = """
                    .styledTable {
                      border-collapse: collapse;
                      text-align: left;
                      line-height: 1.5;
                    }
                    .styledTable thead th {
                      padding: 10px;
                      font-weight: bold;
                      vertical-align: top;
                      color: #369;
                      border-bottom: 3px solid #036;
                    }
                    .styledTable tbody th {
                      width: 150px;
                      padding: 10px;
                      font-weight: bold;
                      vertical-align: top;
                      border-bottom: 1px solid #ccc;
                      background: #f3f6f7;
                    }
                    .styledTable td {
                      width: 350px;
                      padding: 10px;
                      vertical-align: top;
                      border-bottom: 1px solid #ccc;
                    }
                """
        return style_data

    @staticmethod
    def style_hyper():
        style_data ="""
                    .round-button-5 {
                      border-radius: 5px;
                    }
                    .multiple-select-title {
                      border : 1px solid gray;
                      border-bottom : 0px;
                      padding: 0px 10px;
                      background-color : #eeeeee;
                    }
                """
        return style_data

    @staticmethod
    def style_file_loading():
        style_data ="""
                     @keyframes rotate {
                        from {
                            transform: rotate(0deg);
                        }
                        to { 
                            transform: rotate(360deg);
                        }
                    }
                     @-webkit-keyframes rotate {
                        from {
                            -webkit-transform: rotate(0deg);
                        }
                        to { 
                            -webkit-transform: rotate(360deg);
                        }
                    }
                    .load {
                        width: 23px;
                        height: 23px;
                        margin: 10px auto 0;
                        margin-left : 10px;
                        border:solid 5px #8822aa;
                        border-radius: 50%;
                        border-right-color: transparent;
                        border-bottom-color: transparent;
                         -webkit-transition: all 0.5s ease-in;
                        -webkit-animation-name:             rotate; 
                        -webkit-animation-duration:         1.0s; 
                        -webkit-animation-iteration-count:  infinite;
                        -webkit-animation-timing-function: linear;
                             transition: all 0.5s ease-in;
                        animation-name:             rotate; 
                        animation-duration:         1.0s; 
                        animation-iteration-count:  infinite;
                        animation-timing-function: linear; 
                    }
                """
        return style_data

    @staticmethod
    def style_check_box():
        style_data ="""
                    .styled-check-box {
                        height : 20px;
                        overflow : hidden !important;
                    }
                """
        return style_data


#!/usr/bin/env python
# coding: utf-8

# Copyright (c) kyoungjun.
# Distributed under the terms of the Modified BSD License.

"""
TODO: Add module docstring
"""

from ipywidgets import DOMWidget, ValueWidget, register
from traitlets import Unicode, Bool, validate, TraitError

from _frontend import module_name, module_version


@register
class Analysis(DOMWidget, ValueWidget):
    _model_name = Unicode('AnalysisView').tag(sync=True)
    _model_module = Unicode(module_name).tag(sync=True)
    _model_module_version = Unicode(module_version).tag(sync=True)

    _view_name = Unicode('AnalysisView').tag(sync=True)
    _view_module = Unicode(module_name).tag(sync=True)
    _view_module_version = Unicode(module_version).tag(sync=True)

    value = Unicode('example@example.com').tag(sync=True)
    disabled = Bool(False, help="Enable or disable user changes.").tag(sync=True)

    # basic validator for the email value
    @validate('value')
    def _valid_value(self, proposal):
        if proposal['value'].count("@") != 1:
            raise TraitError('Invalid email value: it must contain an "@" character')
        if proposal['value'].count(".") == 0:
            raise TraitError('Invalid email value: it must contain at least one "." character')
        return proposal['value']


#!/usr/bin/env python
# coding: utf-8

# Copyright (c) TmaxBI.
# Distributed under the terms of the Modified BSD License.
import numpy as np
from numpy.random import default_rng
from scipy import stats

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil, StatsUtil
from jupyter_analysis_extension.statistics_operation.statistic_original import StatisticOperation
from jupyter_analysis_extension.statistics_widget.anova.anova_oneway import WidgetANOVAOneWay
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget


class ANOVAOneWay(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        self.op_name = "One-way ANOVA"
        self.sub = [
            {"name": "Dependent Variable(s)", "type": ["N"], "max_count": 0, "min_count": 1},
            {"name": "Grouping Variable", "type": ["C"], "max_count": 1, "min_count": 1}
        ]
        self.option = [
            {"name": "Test Types", "type": "checkbox", "value": ['Welch\'s', 'Fisher\'s']},
        ]
        self.callback = {"refresh": refresh_tab}

        super(ANOVAOneWay, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )

        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        self.set_analyze_text('In progress...')

        # catch invalid data
        option = self.control_data['option']
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        # get analysis data & option
        degree_freedom = len(self.input_df) - 2

        dependent_layout_var_list = list(self.control_data['Dependent Variable(s)'])
        dependent_var_list = []
        for layout_var in dependent_layout_var_list:
            dependent_var_list.append(self.column_value[layout_var])

        grouping_layout_var_list = list(self.control_data['Grouping Variable'])
        group_var = self.column_value[grouping_layout_var_list[0]]
        group_var_unique_list = self.input_df[group_var].unique()

        tests_welchs = option['Test Types']['Welch\'s']
        tests_fishers = option['Test Types']['Fisher\'s']
        tests_options = [tests_welchs, tests_fishers]
        result_widgets = self.get_result_widgets(
            degree_freedom, dependent_var_list, group_var, group_var_unique_list, tests_options)

        CommonUtil.update_result_layout(self.control_panel, result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Dependent Variable(s)', 'Number',
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Grouping Variable', ['N-Category'],
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_option_checkbox(control_data, 'Test Types', 1)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        grouping_var_list = control_data['Grouping Variable']
        group_var = self.column_value[grouping_var_list[0]]
        group_var_unique_count = len(self.input_df[group_var].unique())

        if group_var_unique_count > 20:
            self.set_analyze_text(
                'Grouping variable [' + group_var + '] have too many unique values', "warning")
            return False

        return True

    def get_result_widgets(self, degree_freedom, dependent_var_list, grp_var, group_var_unique_list, tests_options):
        rng = default_rng(1)
        target_column_names = []
        statistic_tests_options = []
        statistic_chart = []
        statistic_chart_title = []
        statistic_result = []
        stratify_size = []
        df = self.input_df
        for dep in dependent_var_list:
            grouped_data = []
            target_column_names.append(dep)

            if len(df[df[dep].notnull()]) > 10000:
                sample = True
            else:
                sample = False

            for grp in group_var_unique_list:
                data = df[(df[grp_var] == grp) & (df[dep].notnull())][dep].values
                size = len(data)
                if size > 2:
                    grouped_data.append(data)
                    stratify_size.append(size)
                else:
                    pass
            stratify_size = list(np.array(stratify_size) * 10000 / (np.sum(stratify_size)))

            if tests_options[0]:
                target_column_names.append('')
                statistic_result.append(StatsUtil.welch_anova(
                    *grouped_data))
                statistic_tests_options.append('Welch\'s')

            if tests_options[1]:
                target_column_names.append('')
                statistic_result.append(stats.f_oneway(
                    *grouped_data))
                statistic_tests_options.append('Fisher\'s')
            target_column_names.pop()

            if sample:
                grouped_data_sample = []
                for data, size in zip(grouped_data, stratify_size):
                    if size > 2:
                        data_sample = rng.choice(data, int(size), replace=False)
                    else:
                        data_sample = rng.choice(data, 2, replace=False)
                    grouped_data_sample.append(data_sample)
            else:
                grouped_data_sample = grouped_data

            boxplot_sns = WidgetANOVAOneWay.get_boxplot_sns(
                grouped_data_sample, group_var_unique_list, dep, grp_var)
            statistic_chart.append(boxplot_sns)
            statistic_chart_title.append('Box Plot (' + str(dep) + ')')
        statistic_result_table = WidgetANOVAOneWay.get_statistic_result_table(
            target_column_names, degree_freedom, statistic_result, statistic_tests_options)

        return CommonWidget.get_accordion_widget([statistic_result_table, *statistic_chart],
                                                 ['Test Result(s)', *statistic_chart_title])

    """
    @staticmethod
    def get_shapiro_widgets(dependent):
        shapiro = stats.shapiro(dependent)
        sub1_shapiro = widgets.HTML(
            value='statistics: {0}, pvalue: {1}'.format(round(sub1_shapiro.statistic, 4),
                                                        round(sub1_shapiro.pvalue, 4)),
            description="sub1's shapiro > ",
            disabled=True
        )
        sub2_shapiro = widgets.HTML(
            value='statistics: {0}, pvalue: {1}'.format(round(sub2_shapiro.statistic, 4),
                                                        round(sub2_shapiro.pvalue, 4)),
            description="sub2's shapiro > ",
            disabled=True
        )
        sharpiro_agg_widget = widgets.VBox([sub1_shapiro, sub2_shapiro])
        return sharpiro_agg_widget

    def Ftest(df, dim, mes):
        dim = 'C(Q(\"' + dim + '\"))'
        mes = 'Q(\"' + mes + '\")'
        equation = mes + ' ~ ' + dim
        try:
            model = ols(equation, data=df).fit()
        except Exception as e:
            return np.nan, 1.
        table = sm.stats.anova_lm(model, typ=2)
        table['mean_sq'] = table[:]['sum_sq'] / table[:]['df']
        table['eta_sq'] = table[:-1]['sum_sq'] / sum(table['sum_sq'])
        table['omega_sq'] = (table[:-1]['sum_sq'] - (table[:-1]['df'] * table['mean_sq'][-1])) / (
                sum(table['sum_sq']) + table['mean_sq'][-1])
        p_value = table.at[dim, 'PR(>F)']
        eta_sq = table.at[dim, 'eta_sq']
        omega_sq = np.abs(table.at[dim, 'omega_sq'])
        effect_size = omega_sq ** (0.4 - (2 * omega_sq)) if omega_sq <= 0.2 else 1.0
        return effect_size, p_value
    """



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) TmaxBI.
# Distributed under the terms of the Modified BSD License.
import ipywidgets as widgets

# Graph
import plotly.graph_objs as go


class WidgetANOVAKruskal:
    @ staticmethod
    def get_statistic_result_table(name, degree_freedom, test_result):
        res_table_info = ''

        for pairindex, item in enumerate(test_result):
            target_degree_freedom = degree_freedom
            statistic_value = round(item.statistic, 3)
            target_degree_freedom = '-'

            res_table_info += """
                        <tr>
                            <td>{0}</td>
                            <td>{1}</td>
                            <td>{2}</td>
                            <td>{3}</td>
                        </tr>
                        """.format(name[pairindex], statistic_value,
                                   target_degree_freedom, round(item.pvalue, 3))

        res_table = widgets.HTML(
            value="""
                <table style="width:100%">
                <tr style="border-bottom: 1px solid black;">
                    <th></th>
                    <th>Statistic</th>
                    <th>df</th>
                    <th>p-value</th>
                </tr>
               {0}
                </table>""".format(res_table_info),
            disabled=True
        )
        statistic_result_table = widgets.VBox([res_table])
        statistic_result_table.add_class('styledTable')

        return statistic_result_table

    @staticmethod
    def get_boxplot_sns(grouped_data, group_val_distinct, dep, grp_var):
        fig = go.FigureWidget()
        for data, index in zip(grouped_data, group_val_distinct):
            fig.add_trace(go.Box(y=data,
                                 name=str(index),
                                 boxpoints='all',
                                 jitter=0.2,
                                 pointpos=-1.8
                                 ))
        fig.layout.yaxis.title = dep
        fig.layout.xaxis.title = grp_var

        fig.update_layout(
            width=500,
            height=500,
        )
        return fig


#!/usr/bin/env python
# coding: utf-8

# Copyright (c) TmaxBI.
# Distributed under the terms of the Modified BSD License.
import ipywidgets as widgets

# Graph
import plotly.graph_objs as go


class WidgetANOVAFriedman:
    @ staticmethod
    def get_statistic_result_table(degree_freedom, test_result):
        res_table_info = ''
        res_table_info += """
                         <tr>
                             <td>{0}</td>
                             <td>{1}</td>
                             <td>{2}</td>
                         </tr>
                         """.format(round(test_result.statistic,2), degree_freedom,test_result.pvalue)

        res_table = widgets.HTML(
            value="""
                      <table style="width:100%">
                      <tr style="border-bottom: 1px solid black;">
                          <th>Statistic</th>
                          <th>df</th>
                          <th>p-value</th>
                      </tr>
                     {0}
                      </table>""".format(res_table_info),
            disabled=True
        )
        statistic_result_table = widgets.VBox([res_table])
        statistic_result_table.add_class('styledTable')
        return statistic_result_table

    @staticmethod
    def get_boxplot_sns(grouped_data, group_val_distinct):
        fig = go.FigureWidget()
        for data, index in zip(grouped_data, group_val_distinct):
            fig.add_trace(go.Box(y=data, name=index))

        fig.update_layout(
            width=500,
            height=500,
        )
        return fig


from scipy import stats

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_original import StatisticOperation
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget
from jupyter_analysis_extension.statistics_widget.t_test.t_test_one_sample import WidgetTTestOneSample
from jupyter_analysis_extension.utils.chart_maker import ChartMaker
from jupyter_analysis_extension.utils.widget_maker import WidgetMaker


class TTestOneSample(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        self.op_name = "T-Test : One Sample"
        self.sub = [
                  {"name": "Dependent Variable(s)", "type": ["N"], "max_count": 0, "min_count": 1},
              ]
        self.option = [
                    {"name": "Assumption", "type": "checkbox", "value": ["Normality test", "Q-Q plot"]},
                     {"name": "Test Types", "type": "checkbox", "value": ["Student\'s", "Wilcoxon signed-rank"]},
                     {"name": "Hypothesis", "type": "numberText-radio", "value": {
                         "numberText": {"name": "Hypothesis", "value": 0},
                         "radio": {"name": None, "value": ["≠ Hypothesis", "> Hypothesis", "< Hypothesis"]}
                     }}
                 ]
        self.callback = {"refresh": refresh_tab}

        super(TTestOneSample, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )

        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1],self.on_analyze_button_clicked )

    def on_analyze_button_clicked(self, widget):

        self.set_analyze_text('In progress...')

        # catch invalid data
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        # get analysis data & option
        degree_freedom = len(self.input_df) - 1

        dependent_var_list = []
        for layout_var in self.control_data['Dependent Variable(s)']:
            dependent_var_list.append(self.column_value[layout_var])

        option = self.control_data['option']

        # Assumption 옵션
        option_assumption = option['Assumption']

        # Test Types 옵션
        option_test_type = option['Test Types']
        tests_students = option['Test Types']['Student\'s']
        tests_wilcoxon = option['Test Types']['Wilcoxon signed-rank']


        # Hypothesis 옵션
        option_hypothesis = option['Hypothesis']
        hypothesis_test_value = option['Hypothesis']['number']
        hypothesis_alternative = option['Hypothesis']['radio']

        if hypothesis_alternative == '≠ Hypothesis':
            hypothesis_alternative = 'two-sided'
        elif hypothesis_alternative == '> Hypothesis':
            hypothesis_alternative = 'greater'
        elif hypothesis_alternative == '< Hypothesis':
            hypothesis_alternative = 'less'

        result_widgets = self.get_result_widgets(
            degree_freedom, dependent_var_list, option_assumption, option_test_type,
            [hypothesis_test_value, hypothesis_alternative])

        CommonUtil.update_result_layout(self.control_panel, result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):
        error_msg = CommonUtil.check_variables_shapes(control_data, 'Dependent Variable(s)', 'Number',
                                                     self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_option_checkbox(control_data, 'Test Types', 1)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        return True

    def get_result_widgets(self, degree_freedom, dependent_var_list, option_assumption, option_test_type, hypothesis_options):
        target_column_names = []
        statistic_tests_options = []
        statistic_chart = []
        statistic_result = []
        accordion_items = []
        accordion_titles = []

        # Assumption 차트 아코디언
        assumption_item = []
        table_header = ['', 'Shapiro-Wilk', 'p-value' , 'Anderson-Darling', 'Critical Value']
        table_body= []
        table_footer ="* Critical Value significance_level is 5.0"
        if option_assumption['Normality test']:
            for item in dependent_var_list:
                shapiro_statistic, shapiro_pvalue = stats.shapiro(self.input_df[item])
                AndersonResult = stats.anderson(self.input_df[item], dist = 'norm')
                table_body.append([item, round(shapiro_statistic, 3), shapiro_pvalue, round(AndersonResult[0], 1), AndersonResult[1][2] ])

            normality_table = WidgetMaker.get_styled_table(table_header, table_body,table_footer)
            assumption_item.append(normality_table)

        if option_assumption['Q-Q plot']:
            for item in dependent_var_list:
                assumption_item.append(ChartMaker.get_qq_plot_plotly(item, self.input_df[item]))

        accordion_assumption = WidgetMaker.get_vertical_box(box_items=assumption_item,
                                     style=dict(display="flex", width="100%",
                                                align_items="center", justify_content="space-around", flex_flow='row wrap'))
        if len(assumption_item) > 0 :
            accordion_items.append(accordion_assumption)
            accordion_titles.append("Assumption")

        # 통계 분석 테이블 아코디언
        table_header = ['', '', 'Statistic', 'df', 'p-value']
        table_body = []

        for item in dependent_var_list:
            target_column_names.append(item)

            if option_test_type['Student\'s']:
                Ttest_1sampResult =stats.ttest_1samp(self.input_df[item], popmean=hypothesis_options[0], alternative=hypothesis_options[1], nan_policy='omit')
                table_body.append([item, 'Student\'s', round(Ttest_1sampResult[0],2), degree_freedom, round(Ttest_1sampResult[1],3)])

            if option_test_type['Wilcoxon signed-rank']:
                first_column = item
                if option_test_type['Student\'s']:
                    first_column = ''
                wilcoxon, pvalue = stats.wilcoxon(
                    self.input_df[item], [hypothesis_options[0] for i in range(len(self.input_df[item]))] ,  alternative=hypothesis_options[1])
                table_body.append(
                    [first_column, 'Wilcoxon signed-rank', round(wilcoxon), '', round(pvalue,3)])

        statistic_analysis_table = WidgetMaker.get_styled_table(table_header, table_body)
        accordion_items.append(statistic_analysis_table)
        accordion_titles.append('Test Result(s)')

        # 데이터 분포 아코디언 생성
        for item in dependent_var_list:
            statistic_chart.append(self.input_df[item])
        accordion_item_title = "Box Plot"
        if len(self.input_df) > 10000:
            # Sampling only 10000 rows
            accordion_item_title += " (Sampling 10000 rows)"
            for index,df in enumerate(statistic_chart):
                statistic_chart[index] = df.copy().sample(n=10000, random_state=1, replace=False)

        accordion_items.append(ChartMaker.get_box_plot_plotly(statistic_chart))
        accordion_titles.append(accordion_item_title)

        return CommonWidget.get_accordion_widget(accordion_items,accordion_titles)



#!/usr/bin/env python
# coding: utf-8

import numpy.ma as ma
# Copyright (c) kyoungjun.
# Distributed under the terms of the Modified BSD License.
from scipy import stats

from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil
from jupyter_analysis_extension.statistics_operation.statistic_original import StatisticOperation
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget
from jupyter_analysis_extension.statistics_widget.t_test.t_test_ind_sample import WidgetTTestIndSample


class TTestIndSample(StatisticOperation):
    def __init__(self, df, column_type_list, refresh_tab, logger):
        self.op_name = "T-Test : Independent Sample"
        self.sub = [
            {"name": "Dependent Variable(s)", "type": ["N"], "max_count": 0, "min_count": 1},
            {"name": "Grouping Variable", "type": ["N"], "max_count": 1, "min_count": 0}
        ]
        self.option = [
            {"name": "Test Types", "type": "checkbox", "value": ['Student\'s', 'Welch\'s', 'Mann-Whitney U']},
            {"name": "Hypothesis", "type": "radio",
             "value": ["Group 1 ≠ Group 2", "Group 1 > Group 2", "Group 1 < Group 2"]}
        ]
        self.callback = {"refresh": refresh_tab}

        super(TTestIndSample, self).__init__(self.op_name, df, column_type_list, refresh_tab, logger)

    def construct_control_panel(self):
        self.control_panel, self.control_data = CommonWidget.make_control_panel(
            df=self.input_df,
            type_list=self.column_type_list,
            title=self.op_name,
            sub=self.sub,
            option=self.option,
            callback=self.callback
        )
        self.column_value = self.control_data["column"]

        self.connect_button_layout(self.control_panel.children[1], self.on_analyze_button_clicked)

    def on_analyze_button_clicked(self, widget):
        self.set_analyze_text('In progress...')

        # catch invalid data
        option = self.control_data['option']
        is_valid = self.validate_input_data(self.control_data)
        if not is_valid:
            return

        # get analysis data & option
        degree_freedom = len(self.input_df) - 2

        dependent_layout_var_list = list(self.control_data['Dependent Variable(s)'])
        dependent_var_list = []
        for layout_var in dependent_layout_var_list:
            dependent_var_list.append(self.column_value[layout_var])

        grouping_layout_var_list = list(self.control_data['Grouping Variable'])
        group_var = self.column_value[grouping_layout_var_list[0]]
        if len(self.input_df[group_var].dropna().unique()) == 1:
            self.input_df[group_var] = self.input_df[group_var].fillna('null')
        else:
            pass
        group_var_unique_list = list(self.input_df[group_var].dropna().unique())

        group_1_data_list = self.input_df[self.input_df[group_var]
                                          == group_var_unique_list[0]]
        group_2_data_list = self.input_df[self.input_df[group_var]
                                          == group_var_unique_list[1]]

        tests_students = option['Test Types']['Student\'s']
        tests_welchs = option['Test Types']['Welch\'s']
        tests_mannwhitneyu = option['Test Types']['Mann-Whitney U']

        hypothesis_alternative = option['Hypothesis']['radio']
        if hypothesis_alternative == 'Group 1 ≠ Group 2':
            hypothesis_alternative = 'two-sided'
        elif hypothesis_alternative == 'Group 1 > Group 2':
            hypothesis_alternative = 'greater'
        elif hypothesis_alternative == 'Group 1 < Group 2':
            hypothesis_alternative = 'less'

        result_widgets = self.get_result_widgets(
            degree_freedom, dependent_var_list, [group_1_data_list, group_2_data_list], group_var_unique_list,
            [tests_students, tests_welchs, tests_mannwhitneyu], [hypothesis_alternative])

        CommonUtil.update_result_layout(self.control_panel, result_widgets)

        self.set_analyze_text('Done.')

    def validate_input_data(self, control_data):
        error_msg = CommonUtil.check_variables_shapes(control_data, 'Dependent Variable(s)', 'Number',
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_option_checkbox(control_data, 'Test Types', 1)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        error_msg = CommonUtil.check_variables_shapes(control_data, 'Grouping Variable',
                                                      ['2-Category'],
                                                      self.column_type_list, self.column_value, self.sub)
        if error_msg is not None:
            self.set_analyze_text(error_msg, 'warning')
            return False

        return True

    def get_result_widgets(self, degree_freedom, dependent_var_list, grouped_df, group_var_unique_list, tests_options,
                           hypothesis_options):
        target_column_names = []
        statistic_tests_options = []
        statistic_chart = []
        statistic_result = []

        for item in dependent_var_list:
            chart_data = []
            target_column_names.append(item)
            chart_data.append(grouped_df[0][item])
            chart_data.append(grouped_df[1][item])
            if tests_options[0]:
                test_result = stats.ttest_ind(
                    grouped_df[0][item], grouped_df[1][item], equal_var=True, nan_policy='omit',
                    alternative=hypothesis_options[0])
                if ma.is_masked(test_result.statistic):
                    self.set_analyze_text('Invalid input fot t-test (Complete Bias)', 'warning')
                else:
                    target_column_names.append('')
                    statistic_result.append(test_result)
                    statistic_tests_options.append('Student\'s')

            if tests_options[1]:
                test_result = stats.ttest_ind(
                    grouped_df[0][item], grouped_df[1][item], equal_var=False, nan_policy='omit',
                    alternative=hypothesis_options[0])
                if ma.is_masked(test_result.statistic):
                    self.set_analyze_text('Invalid input fot t-test (Complete Bias)', 'warning')
                else:
                    target_column_names.append('')
                    statistic_result.append(test_result)
                    statistic_tests_options.append('Welch\'s')

            if tests_options[2]:
                if len(grouped_df[0][item].dropna()) < 1 or len(grouped_df[1][item].dropna()) < 1:
                    self.set_analyze_text('Invalid input fot t-test (Complete Bias)', 'warning')
                else:
                    test_result = stats.mannwhitneyu(
                        grouped_df[0][item].dropna(), grouped_df[1][item].dropna(),
                        alternative=hypothesis_options[0])
                    target_column_names.append('')
                    statistic_result.append(test_result)
                    statistic_tests_options.append('Mann-Whitney U')

            statistic_chart.append(chart_data)
            target_column_names.pop()

        statistic_result_table = WidgetTTestIndSample.get_statistic_result_table(
            target_column_names, degree_freedom, statistic_result, statistic_tests_options)

        plot_names = ['Box Plot (' + name + ')' for name in dependent_var_list]
        if len(self.input_df) > 10000:
            # Sampling only 10000 rows
            for index, name in enumerate(plot_names):
                plot_names[index] += name + " (Sampling 10000 rows)"
            for index, df in enumerate(statistic_chart):
                statistic_chart[index] = df.copy().sample(n=10000, random_state=1, replace=False)

        box_plot_plotly = WidgetTTestIndSample.get_box_plot_plotly(
            statistic_chart, dependent_var_list, group_var_unique_list)
        plot_names = ['Box Plot (' + name + ')' for name in dependent_var_list]
        return CommonWidget.get_accordion_widget([statistic_result_table] + box_plot_plotly,
                                                 ['Test Result(s)'] + plot_names)



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) kyoungjun.
# Distributed under the terms of the Modified BSD License.

# GUI

import ipywidgets as widgets
import qgrid
from IPython import get_ipython
from IPython.display import display

from jupyter_analysis_extension.utils.logger import OutputWidgetHandler
#from nbformat import v4 as nbf
# from nbformat import current as nbf
import nbformat
from nbformat import v4 as nbf

# Hyderdata

try:
    import hyper
except ModuleNotFoundError:
    pass

# Common Class
from jupyter_analysis_extension.utils.widget_style import WidgetStyle
from jupyter_analysis_extension.statistics_widget.common_widget import CommonWidget
from jupyter_analysis_extension.statistics_operation.common_util import CommonUtil

# Statistic basic
from jupyter_analysis_extension.statistics_operation.basic.basic_analysis import BasicAnalysis

# Statistic ANOVA
from jupyter_analysis_extension.statistics_operation.anova.anova_oneway import ANOVAOneWay
from jupyter_analysis_extension.statistics_operation.anova.anova_ancova import ANOVAANCOVA
from jupyter_analysis_extension.statistics_operation.anova.anova_kruskal import ANOVAKruskal
from jupyter_analysis_extension.statistics_operation.anova.anova_rm import ANOVARM

# Statistic Regression
from jupyter_analysis_extension.statistics_operation.regression.general_linear_regression import GeneralLinearRegression
from jupyter_analysis_extension.statistics_operation.regression.logistic_regression import LogisticRegression

# Statistic T-test
from jupyter_analysis_extension.statistics_operation.t_test.t_test_one_sample import TTestOneSample
from jupyter_analysis_extension.statistics_operation.t_test.t_test_ind_sample import TTestIndSample
from jupyter_analysis_extension.statistics_operation.t_test.t_test_pair_sample import TTestPairSample

# Statistic Frequencies
from jupyter_analysis_extension.statistics_operation.frequencies.frequencies_chi_square import FrequenciesChiSquare
from jupyter_analysis_extension.statistics_operation.frequencies.frequencies_mcnemar import FrequenciesMcnemar

import logging
from jupyter_analysis_extension.utils.file_selecter import FileSelector

OPERATION_LIST = {
    "Basic" : ["Basic : Analysis"],
    "T-test": ['T-Test : One Sample', 'T-Test : Independent Sample', 'T-Test : Paired Sample'],
    "ANOVA": ['One-way ANOVA', 'ANOVA (Kruskal-Wallis)', 'Repeated Measure ANOVA', 'ANOVA & ANCOVA'],
    "Regression": ['Linear Regression', 'Logistic Regression'],
    "Frequencies": ['Frequencies : Chi-Square', "Frequencies : McNemar"]
}


class WidgetGuide:
    def __init__(self, mode=None, data=None, debug=False):
        WidgetStyle.style_default()

        self.output = widgets.Output()

        self.tab = widgets.Tab()
        self.tab_count = 0
        self.tab_titles = []
        self.tab_children = []

        self.active_op_class = []
        self.combobox = None

        self.input_df = None
        self.column_type_dict = None
        self.mode = mode

        # Logging
        if debug:
            self.logger = logging.getLogger(__name__)
            handler = OutputWidgetHandler()
            handler.setFormatter(logging.Formatter('%(asctime)s  - [%(levelname)s] %(message)s'))
            self.logger.addHandler(handler)
            self.logger.setLevel(logging.INFO)
            handler.show_logs()

        self.FileSelector = FileSelector(mode, data, self.set_main_frame, self.logger)
        display(self.FileSelector.get_file_selector_container())

    def set_main_frame(self, df):
        # start_time = time.time()
        # self.reset_widget()
        # self.logger.info(time.time() - start_time)
        self.input_df = df

        # Sampling only 10000 rows
        max_len = 10000

        sampled_df = self.input_df.copy()
        if len(self.input_df) > max_len:
            sampled_df = sampled_df.sample(n=max_len, random_state=1, replace=False)

        qgrid_widget = qgrid.show_grid(sampled_df, show_toolbar=False)

        self.column_type_dict = self.get_type_dict()
        type_widget = self.get_type_dict_accordion(self.column_type_dict)

        qgrid_widget_width_label = CommonWidget.get_widget_with_label("Table Info (10,000 Rows Sampled)", qgrid_widget)

        menu_widgets = []
        for key in OPERATION_LIST.keys():
            menu_widget = CommonWidget.get_popup_menu(
                op_category=key,
                items=OPERATION_LIST[key],
                click_event=self.on_append_tab)
            menu_widgets.append(menu_widget)

        menu_layout = CommonWidget.get_hbox(menu_widgets, justify_content="flex-start")
        menu_layout.add_class("icon_menu")
        menu_layout_with_label = CommonWidget.get_widget_with_label("Operation Type", menu_layout)
        menu_layout_with_label.add_class("icon_menu")
        self.tab_children = [widgets.VBox(
            [menu_layout_with_label,qgrid_widget_width_label, type_widget])]

        self.tab_titles = ['Main']

        self.tab.children = self.tab_children
        self.tab.set_title(self.tab_count, self.tab_titles[self.tab_count])
        self.tab_count += 1

        display(self.tab)

    def reset_widget(self):
        self.tab.close_all()
        self.output.clear_output()
        self.output.close_all()
        self.__init__()

    def refresh_tab(self, target_op):
        target_widget = self.get_operation_widget(target_op)
        tab_children = list(self.tab.children)
        tab_children[self.tab.selected_index] = target_widget
        self.tab.children = tuple(tab_children)

    def on_func_selected(self, value):
        if value['name'] == 'value' and value['new'] != {} and value['new'] != "":
            if value['owner'].options[0] == value['new']:
                return
            op_name = value['new']
            self.on_append_tab(op_name)
            value['owner'].index = 0

    def on_append_tab(self, op_name):
        if op_name not in self.tab_titles:
            self.tab_children.append(self.get_operation_widget(op_name))
            self.tab_titles.append(op_name)
            self.tab.children = self.tab_children
            self.tab.set_title(
                self.tab_count, self.tab_titles[self.tab_count])
            self.tab.selected_index = self.tab_count
            self.tab_count += 1
        else:
            self.tab.selected_index = self.tab_titles.index(op_name)

    def on_type_changed(self, change):
        if change['type'] == 'change' and change['name'] == 'value':
            self.column_type_dict[change['owner'].description] = change['new']
            for op_class in self.active_op_class:
                op_class.set_refresh_text(
                    str("Column types changed! Please refresh the tab (e.g., " + change['owner'].description)
                    + "'s type -> " + str(change['new']) + ").")
        return

    def get_type_dict(self):
        column_type_dict = {}
        for column in self.input_df.columns:
            abstract_type = CommonUtil.get_abstract_type(
                self.input_df[column])
            column_type_dict[column] = abstract_type
        return column_type_dict

    def get_type_dict_accordion(self, column_type_dict):
        accordion_widget_children = []
        for key in list(column_type_dict.keys()):
            avail_type_list = CommonUtil.get_avail_type_list(
                self.input_df[key])
            dropdown_with_label = CommonWidget.get_dropdown_with_description(
                key, avail_type_list, column_type_dict[key])
            accordion_widget_children.append(dropdown_with_label)
            dropdown_with_label.observe(self.on_type_changed)

        accordion_widget = widgets.Accordion(
            children=[widgets.VBox(accordion_widget_children,
                                   layout=widgets.Layout(margin="0px", width="100%", display="flex",
                                                         flex_flow="wrap"))], selected_index=None,
            layout=widgets.Layout(margin="0px", width="100%"))
        accordion_widget.set_title(0, "Controls of Column Types")
        return accordion_widget

    def get_operation_widget(self, op_name):
        result_widget = None
        target_op = None
        if op_name == "Basic : Analysis":
            self.logger.info("entered")
            nb = nbf.new_notebook()
            cell = nbf.new_code_cell("ss")
            nb.cells.extend(cell)
            with open('Untitled_guide.ipynb', 'w') as f:
                nbformat.write(nb, f)
        else:
            result_widget = widgets.Text(description="New Tab")

        result_widget = target_op.get_tab_widget()

        if target_op is not None:
            self.active_op_class.append(target_op)

        return result_widget
